@@ -1,6 +1,6 @@
 /****************************************************************************************
 *  YAROCK                                                                               *
-*  Copyright (c) 2010-2014 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
+*  Copyright (c) 2010-2016 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
 *                                                                                       *
 *  This program is free software; you can redistribute it and/or modify it under        *
 *  the terms of the GNU General Public License as published by the Free Software        *
@@ -21,16 +21,16 @@
 
 #include <QtCore>
 
-/*******************************************************************************
-    Extension
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* Extension                                                                  */
+/* ---------------------------------------------------------------------------*/ 
 const QStringList m3u_extension  = QStringList() << "m3u" << "m3u8";
 const QStringList pls_extension  = QStringList() << "pls";
 const QStringList xspf_extension = QStringList() << "xspf";
 
-/*******************************************************************************
-    Internals functions
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* Internals functions                                                        */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr>  readM3uPlaylist(QIODevice* device, const QDir& playlist_dir=QDir() );
 QList<MEDIA::TrackPtr>  readPlsPlaylist(QIODevice* device, const QDir& playlist_dir=QDir() );
 QList<MEDIA::TrackPtr>  readXspfPlaylist(QIODevice* device, const QDir& playlist_dir=QDir() );
@@ -39,34 +39,29 @@ void saveM3uPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
 void savePlsPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::TrackPtr> list);
 void saveXspfPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::TrackPtr> list);
 
-/*
-********************************************************************************
-*                                                                              *
-*    PlaylistFromFile                                                          *
-*                                                                              *
-********************************************************************************
-*/
+/* ---------------------------------------------------------------------------*/
+/* MEDIA::PlaylistFromFile                                                    */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr> MEDIA::PlaylistFromFile(const QString& filename)
 {
-    Debug::debug() << "[MEDIA] PlaylistFromFile--> Start "  << QTime::currentTime().second() << ":" << QTime::currentTime().msec();
-
-    //Debug::debug() << "[MEDIA] PlaylistFromFile";
     QList<MEDIA::TrackPtr>   list;
 
-    // Open the file
+    /* get file info */
     QFileInfo info(filename);
+    const QString extension = info.suffix().toLower();
+    const QDir playlist_dir = QDir(info.absoluteDir());
+
+    Debug::debug() << "  [MEDIA] PlaylistFromFile :"  << filename;
+    
+    /* test opening file */
     QFile file(filename);
 
-
     if (!file.open(QFile::ReadOnly)) {
-      Debug::warning() << "Failed to open read only file " << filename;
+      Debug::warning() << "  [MEDIA] Failed to open read only file " << filename;
       return list;
     }
 
-    QString extension = info.suffix().toLower();
-    QDir playlist_dir = QDir(info.absoluteDir());
-
-    // choose parser
+    /* choose parser */
     if( m3u_extension.contains(extension) )
       list = readM3uPlaylist( &file, playlist_dir);
     else if( pls_extension.contains(extension) )
@@ -74,27 +69,23 @@ QList<MEDIA::TrackPtr> MEDIA::PlaylistFromFile(const QString& filename)
     else if( xspf_extension.contains(extension ))
       list = readXspfPlaylist( &file, playlist_dir );
 
-    Debug::debug() << "[MEDIA] PlaylistFromFile : read " << list.size() << " entries";
+    Debug::debug() << "  [MEDIA] PlaylistFromFile : read " << list.size() << " entries";
     
-    Debug::debug() << "[MEDIA] PlaylistFromFile--> End "  << QTime::currentTime().second() << ":" << QTime::currentTime().msec();
     return list;
 }
 
-/*
-********************************************************************************
-*                                                                              *
-*    PlaylistFromBytes                                                         *
-*                                                                              *
-********************************************************************************
-*/
+
+/* ---------------------------------------------------------------------------*/
+/* MEDIA::PlaylistFromBytes                                                   */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr> MEDIA::PlaylistFromBytes(QByteArray& bytes)
 {
-    Debug::debug() << "MEDIA::PlaylistFromBytes";
+    Debug::debug() << "  [MEDIA]  PlaylistFromBytes";
     QList<MEDIA::TrackPtr>   list;
 
     QBuffer buffer(&bytes);
     if (!buffer.open(QFile::ReadOnly)) {
-      Debug::debug() << "Failed to open data";
+      Debug::debug() << "  [MEDIA] Failed to open data";
       return list;
     }
 
@@ -109,17 +100,18 @@ QList<MEDIA::TrackPtr> MEDIA::PlaylistFromBytes(QByteArray& bytes)
     else
     {
       // try m3u style
-      Debug::warning() << "MEDIA::PlaylistFromBytes unknown format, trying m3u playlist...";
+      Debug::warning() << "[MEDIA] PlaylistFromBytes unknown format, trying m3u playlist...";
       list = readM3uPlaylist( &buffer ) ;
     }
+    Debug::debug() << "  [MEDIA]  PlaylistFromBytes END";
 
     return list;
 }
 
 
-/*******************************************************************************
-    M3U playlist read
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* M3U playlist read                                                          */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr>  readM3uPlaylist(QIODevice* device, const QDir& playlist_dir )
 {
     QList<MEDIA::TrackPtr>   list;
@@ -173,7 +165,7 @@ QList<MEDIA::TrackPtr>  readM3uPlaylist(QIODevice* device, const QDir& playlist_
       }
       else if( !line.isEmpty() )
       {
-        //Debug::debug() << "---- readM3uPlaylist -> line " << line << "\n";
+        //Debug::debug() << "  [MEDIA] readM3uPlaylist -> line " << line << "\n";
         MEDIA::TrackPtr track = MEDIA::TrackPtr(new MEDIA::Track());
 
         //! Find the Track location
@@ -183,7 +175,7 @@ QList<MEDIA::TrackPtr>  readM3uPlaylist(QIODevice* device, const QDir& playlist_
               track->setType(TYPE_STREAM);
               track->id          = -1;
               track->url         = url.toString();
-              track->name        = title.isEmpty() ? url.toString() : title;
+              track->extra["station"]  = title.isEmpty() ? QString() : title;
               track->isFavorite  = false;
               track->isPlaying   = false;
               track->isBroken    = false;
@@ -236,93 +228,93 @@ QList<MEDIA::TrackPtr>  readM3uPlaylist(QIODevice* device, const QDir& playlist_
   return list;
 }
 
-/*******************************************************************************
-    .pls playlist read
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* PLS playlist read                                                          */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr>  readPlsPlaylist(QIODevice* device, const QDir& playlist_dir )
 {
-    Debug::debug() << "start readPlsPlaylist";
+    QMap<int, MEDIA::TrackPtr> tracks;
+  
     QList<MEDIA::TrackPtr> list;
 
-    MEDIA::TrackPtr mi = MEDIA::TrackPtr(0);
+    QRegExp n_re("\\d+$");
 
-    while (!device->atEnd()) {
+    while ( !device->atEnd() )
+    {
       QString line = QString::fromUtf8(device->readLine()).trimmed();
-      int equals = line.indexOf('=');
-      QString key = line.left(equals).toLower();
+      
+      int equals    = line.indexOf('=');
+      QString key   = line.left(equals).toLower();
       QString value = line.mid(equals + 1);
 
+      n_re.indexIn(key);
+      int n = n_re.cap(0).toInt();
+      if( !tracks.contains(n) && n > 0 )
+      {
+        tracks[n] = MEDIA::TrackPtr(new MEDIA::Track());
+        
+        tracks.value(n)->setType(TYPE_STREAM);
+        tracks.value(n)->id          = -1;
+        tracks.value(n)->isFavorite  = false;
+        tracks.value(n)->isPlaying   = false;
+        tracks.value(n)->isBroken    = false;
+        tracks.value(n)->isPlayed    = false;
+        tracks.value(n)->isStopAfter = false;
+      }
+    
+      //Debug::debug() << "  [MEDIA] readPlsPlaylist -> key:" << key << " value:" << value;
+      
       if (key.startsWith("file"))
       {
-        mi = MEDIA::TrackPtr(new MEDIA::Track());
-        list.append(mi);
-
         //! Find the Track location
-        if (value.contains(QRegExp("^[a-z]+://"))) {
+        if (value.contains(QRegExp("^[a-z]+://")))
+        {
           QUrl url(value);
           if (url.isValid()) {
-              // Debug::debug() << "---- readPlsPlaylist -> url.isValid()" << url;
-              mi->setType(TYPE_STREAM);
-              mi->id          = -1;
-              mi->url         = value;
-              mi->name        = value;
-              mi->isFavorite  = false;
-              mi->isPlaying   = false;
-              mi->isBroken    = false;
-              mi->isPlayed    = false;
-              mi->isStopAfter = false;
+              //Debug::debug() << "  [MEDIA] readPlsPlaylist -> url.isValid()" << url;
+              tracks.value(n)->setType(TYPE_STREAM);
+              tracks.value(n)->url         = value;
           }
         }
-        else {
+        else 
+        {
           QString file_path = value;
 
           file_path = QDir::fromNativeSeparators(file_path);
 
           // Make the path absolute
           if (!QDir::isAbsolutePath(file_path))
-            file_path = playlist_dir.absoluteFilePath(file_path);
+              file_path = playlist_dir.absoluteFilePath(file_path);
 
           // Use the canonical path
           if (QFile::exists(file_path))
-            file_path = QFileInfo(file_path).canonicalFilePath();
+              file_path = QFileInfo(file_path).canonicalFilePath();
 
-          mi->setType(TYPE_TRACK);
-          mi->id          =  -1;
-          mi->url         =  file_path;
-          mi->isPlaying   =  false;
-          mi->isBroken    =  QFile::exists(file_path) ? false : true;
-          mi->isPlayed    =  false;
-          mi->isStopAfter =  false;
+          tracks.value(n)->setType(TYPE_TRACK);
+          tracks.value(n)->url         =  file_path;
+          tracks.value(n)->isBroken    =  QFile::exists(file_path) ? false : true;
         }
       } // key is filename
       else if (key.startsWith("title"))
       {
-        if(mi->type() == TYPE_TRACK)
-          mi->title = value;
-        else
-          mi->name = value;
+        tracks.value(n)->title = value;
       }
       else if (key.startsWith("length"))
       {
-        if(mi->type() == TYPE_TRACK)
-          mi->duration = value.toInt();
+        if( value.toInt() > 0 )
+            tracks.value(n)->duration = value.toInt();
       }
     } // fin while
 
-    Debug::debug() << "end readPlsPlaylist";
-
-    return list;
+    return tracks.values();
 }
 
 
-
-/*******************************************************************************
-    .xspf playlist read
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* XSPF playlist read                                                         */
+/* ---------------------------------------------------------------------------*/ 
 QList<MEDIA::TrackPtr>  readXspfPlaylist(QIODevice* device, const QDir& playlist_dir )
 {
-    Debug::debug() << "[MEDIA] readXspfPlaylist";
-
     QList<MEDIA::TrackPtr>  list;
 
     QXmlStreamReader xml(device);
@@ -345,24 +337,23 @@ QList<MEDIA::TrackPtr>  readXspfPlaylist(QIODevice* device, const QDir& playlist
       
       if (xml.isStartElement() && xml.name() == "track")
       {
-        //Debug::debug() << "---- readXspfPlaylist -> NEW Track ";
+        //Debug::debug() << "  [MEDIA] readXspfPlaylist -> NEW Track ";
         mi = MEDIA::TrackPtr(new MEDIA::Track());
       }
       else if (xml.isStartElement() && xml.name() == "location")
       {
             QString file_path = QString(xml.readElementText());
 
-            //Debug::debug() << "---- readXspfPlaylist -> Find the Track location" << file_path;
+            //Debug::debug() << "  [MEDIA] readXspfPlaylist -> Find the Track location" << file_path;
             if (!MEDIA::isLocal(file_path)) {
               QUrl url(file_path);
               if (url.isValid()) {
 
-                //Debug::debug() << "---- readXspfPlaylist -> it's an url";
+                //Debug::debug() << "  [MEDIA] readXspfPlaylist -> it's an url";
                 if(mi) {
                   mi->setType(TYPE_STREAM);
                   mi->id          = -1;
                   mi->url         = file_path;
-                  mi->name        = file_path;
                   mi->isFavorite  = false;
                   mi->isPlaying   = false;
                   mi->isBroken    = false;
@@ -372,20 +363,20 @@ QList<MEDIA::TrackPtr>  readXspfPlaylist(QIODevice* device, const QDir& playlist
               }
             }
             else {
-              //Debug::debug() << "---- readXspfPlaylist -> it's a local file";
+              //Debug::debug() << "  [MEDIA] readXspfPlaylist -> it's a local file";
 
               file_path = QDir::fromNativeSeparators(file_path);
-              //Debug::debug() << "---- readXspfPlaylist -> file_path" << file_path;
+              //Debug::debug() << "  [MEDIA] readXspfPlaylist -> file_path" << file_path;
 
               // Make the path absolute
               if (!QDir::isAbsolutePath(file_path))
                 file_path = playlist_dir.absoluteFilePath(file_path);
-              //Debug::debug() << "---- readXspfPlaylist -> file_path" << file_path;
+              //Debug::debug() << "  [MEDIA] readXspfPlaylist -> file_path" << file_path;
 
               // Use the canonical path
               if (QFile::exists(file_path))
                 file_path = QFileInfo(file_path).canonicalFilePath();
-              //Debug::debug() << "---- readXspfPlaylist -> file_path" << file_path;
+              //Debug::debug() << "  [MEDIA] readXspfPlaylist -> file_path" << file_path;
 
               if(mi) {
                 mi->setType(TYPE_TRACK);
@@ -400,54 +391,59 @@ QList<MEDIA::TrackPtr>  readXspfPlaylist(QIODevice* device, const QDir& playlist
       } // end location
       else if (xml.isStartElement() && xml.name() == "title")
       {
-          if(mi->type() == TYPE_TRACK)
+         if(mi->type() == TYPE_TRACK)
             mi->title = QString(xml.readElementText());
-          else
-            mi->name = QString(xml.readElementText());
+         else
+            mi->extra["station"] = QString(xml.readElementText());
+      }
+      else if (xml.isStartElement() && xml.name() == "creator")
+      {
+          mi->artist = QString(xml.readElementText());
+      }
+      else if (xml.isStartElement() && xml.name() == "album")
+      {
+          mi->album = QString(xml.readElementText());
       }
       else if (xml.isStartElement() && xml.name() == "category")
       {
           if(mi->type() == TYPE_STREAM)
-            mi->categorie = QString(xml.readElementText());
+            mi->genre = QString(xml.readElementText());
       }
       else if (xml.isEndElement() && xml.name() == "track")
       {
-        //Debug::debug() << "---- readXspfPlaylist -> list.append(mi)" << mi;
+        //Debug::debug() << "  [MEDIA] readXspfPlaylist -> list.append(mi)" << mi;
         if(mi)
           list.append(mi);
         mi = MEDIA::TrackPtr(0);
       }
     }  // End while xml end
 
-    //Debug::debug() << "readXspfPlaylist -> END OK";
+    //Debug::debug() << "  [MEDIA] readXspfPlaylist -> END OK";
 
     return list;
 }
 
 
-/*
-********************************************************************************
-*                                                                              *
-*    PlaylistToFile                                                            *
-*                                                                              *
-********************************************************************************
-*/
+/* ---------------------------------------------------------------------------*/
+/* MEDIA::PlaylistToFile                                                      */
+/* ---------------------------------------------------------------------------*/ 
 void MEDIA::PlaylistToFile(const QString& filename, QList<MEDIA::TrackPtr> list)
 {
-    Debug::debug() << "[MEDIA] PlaylistToFile track count :" << list.size();
-
-    // Open the file
-    QFileInfo info(filename);
+    Debug::debug() << "  [MEDIA] PlaylistToFile track count :" << list.size();
 
+    /* test opening file */
     QFile file(filename);
     if (!file.open(QFile::WriteOnly)) {
       Debug::warning() << "Failed to open file for writing" << filename;
       return;
     }
 
-    QString extension = info.suffix().toLower();
-    QDir playlist_dir = QDir(info.absoluteDir());
+    /* get file info */
+    QFileInfo info(filename);
+    const QString extension = info.suffix().toLower();
+    const QDir playlist_dir = QDir(info.absoluteDir());
 
+    /* choose playlist parser */
     if( m3u_extension.contains(extension) )
       saveM3uPlaylist(&file, playlist_dir, list);
     else if( pls_extension.contains(extension) )
@@ -458,9 +454,9 @@ void MEDIA::PlaylistToFile(const QString& filename, QList<MEDIA::TrackPtr> list)
     file.close();
 }
 
-/*******************************************************************************
-    m3u playlist save
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* M3U playlist save                                                          */
+/* ---------------------------------------------------------------------------*/ 
 void saveM3uPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::TrackPtr> list)
 {
     device->write("#EXTM3U\n");
@@ -475,7 +471,7 @@ void saveM3uPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
       if (media->type() == TYPE_TRACK ) {
         info = QString("#EXTINF:%1,%2 - %3\n").arg(QString::number(media->duration))
                                               .arg(media->artist)
-                                              .arg(media->album);
+                                              .arg(media->title);
         device->write(info.toUtf8());
 
         //! Get the path to MediaItem relative to the Playlist File directory
@@ -485,7 +481,7 @@ void saveM3uPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
         info = QString("#EXTINF:\n");
         info = QString("#EXTINF:%1,%2 - %3\n").arg(0)
                                               .arg("")
-                                              .arg(media->name);
+                                              .arg(media->extra["station"].toString());
 
         device->write(info.toUtf8());
         media_path = media->url;
@@ -497,9 +493,9 @@ void saveM3uPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
 }
 
 
-/*******************************************************************************
-    .pls playlist save
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* PLS playlist save                                                          */
+/* ---------------------------------------------------------------------------*/ 
 void savePlsPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::TrackPtr> list)
 {
     QTextStream stream(device);
@@ -513,8 +509,9 @@ void savePlsPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
       if (!media) continue;
       if (media->type() != TYPE_TRACK && media->type() != TYPE_STREAM) continue;
 
-      // write metadata
-      if (media->type() == TYPE_TRACK ) {
+      /* TYPE_TRACK */
+      if (media->type() == TYPE_TRACK ) 
+      {
         QString media_path = media->url;
         media_path = playlist_dir.relativeFilePath(media_path).toUtf8();
 
@@ -523,9 +520,10 @@ void savePlsPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
         stream << "Length" << n << "=" << media->duration << endl;
         ++n;
       }
-      else { // TYPE_STREAM
+      else /* TYPE_STREAM */
+      { 
         stream << "File" << n << "=" << QString(media->url).toUtf8() << endl;
-        stream << "Title" << n << "=" << QString(media->name).toUtf8() << endl;
+        stream << "Title" << n << "=" << QString(media->extra["station"].toString()).toUtf8() << endl;
         stream << "Length" << n << "=" << endl;
         ++n;
       }
@@ -533,10 +531,9 @@ void savePlsPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::T
 }
 
 
-
-/*******************************************************************************
-    .xspf playlist save
-*******************************************************************************/
+/* ---------------------------------------------------------------------------*/
+/* XSPF playlist save                                                         */
+/* ---------------------------------------------------------------------------*/ 
 void saveXspfPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::TrackPtr> list)
 {
     QXmlStreamWriter xml(device);
@@ -556,7 +553,8 @@ void saveXspfPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::
 
       QString media_path;
       // write metadata
-      if (media->type() == TYPE_TRACK ) {
+      if (media->type() == TYPE_TRACK )
+      {
         media_path = playlist_dir.relativeFilePath(media->url).toUtf8();
 
         xml.writeTextElement("location", media_path);
@@ -565,17 +563,18 @@ void saveXspfPlaylist(QIODevice* device, const QDir& playlist_dir, QList<MEDIA::
         xml.writeTextElement("album", media->album);
         xml.writeTextElement("duration", QString::number(media->duration* 1000));
       }
-      else { // TYPE_STREAM
+      else /* TYPE_STREAM */ 
+      { 
         media_path = media->url;
 
         xml.writeTextElement("location", media_path);
-        xml.writeTextElement("title", media->name);
+        xml.writeTextElement("title", media->extra["station"].toString());
         
-        if( !media->categorie.isEmpty() )
+        if( !media->genre.isEmpty() )
         {
           xml.writeStartElement("extension");
           xml.writeAttribute("application", "yarock");
-          xml.writeTextElement("category",  media->categorie);
+          xml.writeTextElement("category",  media->genre);
           xml.writeEndElement(); //extension  
         }
       }
