@@ -1,6 +1,6 @@
 /****************************************************************************************
 *  YAROCK                                                                               *
-*  Copyright (c) 2010-2014 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
+*  Copyright (c) 2010-2016 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
 *                                                                                       *
 *  This program is free software; you can redistribute it and/or modify it under        *
 *  the terms of the GNU General Public License as published by the Free Software        *
@@ -20,6 +20,10 @@
 #include "core/mediaitem/mediaitem.h"
 #include "core/player/engine_base.h"
 #include "core/player/engine.h"
+#include "covers/covercache.h"
+
+#include "widgets/ratingwidget.h"
+#include "core/database/database_cmd.h"
 
 #include "online/lastfm.h"
 
@@ -30,6 +34,15 @@
 
 
 
+#if QT_VERSION >= 0x050000
+#include <QtConcurrent>
+#endif
+
+#include <QtCore>
+#include <QScrollBar>
+#include <QPainter>
+#include <QDragMoveEvent>
+
 bool SortByRowUptoDown(const QModelIndex& a, const QModelIndex& b) {
   return a.row() < b.row();
 }
@@ -39,8 +52,8 @@ bool SortByRowDowntoUp(const QModelIndex& a, const QModelIndex& b) {
 }
 
 
-const int cst_minHeightNormal   = 20;
 const int cst_minHeightExtended = 40;
+
 /*
 ********************************************************************************
 *                                                                              *
@@ -50,7 +63,7 @@ const int cst_minHeightExtended = 40;
 */
 PlaylistView::PlaylistView(QWidget *parent, PlayqueueModel* model) : QListView(parent)
 {
-    Debug::debug() << "--- PlaylistView creation";
+    //Debug::debug() << "    [PlaylistView] creation";
 
     //! Gui
     this->setFrameShape(QFrame::NoFrame);
@@ -71,24 +84,23 @@ PlaylistView::PlaylistView(QWidget *parent, PlayqueueModel* model) : QListView(p
     this->setDropIndicatorShown(true);
     this->setDragDropMode(QAbstractItemView::DragDrop);
     this->setMouseTracking(true);
+    this->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 
+    
     //! vertical scrolbar setup
     this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
 
-    //! connection
-    connect(ACTIONS()->value(LASTFM_LOVE), SIGNAL(triggered()), this, SLOT(slot_lastfm_love()));
-    connect(ACTIONS()->value(PLAYQUEUE_REMOVE_ITEM), SIGNAL(triggered()), this, SLOT(removeSelected()));
-    connect(ACTIONS()->value(PLAYQUEUE_JUMP_TO_TRACK), SIGNAL(triggered()), this, SLOT(jumpToCurrentlyPlayingTrack()));
-    connect(this, SIGNAL(doubleClicked(const QModelIndex &)), SLOT(slot_itemActivated(const QModelIndex &)));
-    
     //! model
     m_model = model;
     this->setModel(m_model->proxy());
 
     //! Delegate
     m_delegate = new PlaylistDelegate(this, m_model);
+    m_delegate->setView(this);
     this->setItemDelegate(m_delegate);
 
+    //! connection
+    connect(this, SIGNAL(doubleClicked(const QModelIndex &)), SLOT(slot_itemActivated(const QModelIndex &)));
     connect(m_model, SIGNAL(needSelectionAfterMove(QList<MEDIA::TrackPtr>)), SLOT(selectItems(QList<MEDIA::TrackPtr>)));
     connect(m_model, SIGNAL(modelCleared()), SLOT(slot_model_cleared()));
 
@@ -102,7 +114,7 @@ PlaylistView::PlaylistView(QWidget *parent, PlayqueueModel* model) : QListView(p
 // gestion du doucle click sur un item
 void PlaylistView::slot_itemActivated(const QModelIndex &index)
 {
-    //Debug::debug() << "--- PlaylistView::slot_itemActivated" << index.row();
+    //Debug::debug() << "    [PlaylistView] slot_itemActivated" << index.row();
     QModelIndex source_idx = m_model->proxy()->mapToSource(index);
     if (m_model->rowExists(source_idx.row()))
     {
@@ -190,8 +202,7 @@ const MEDIA::TrackPtr PlaylistView::firstSelectedTrack()
 *******************************************************************************/
 void PlaylistView::removeSelected()
 {
-    //Debug::debug() << "--- PlaylistView::removeSelected";
-
+    //Debug::debug() << "    [PlaylistView] removeSelected";
     QItemSelection selection = selectionModel()->selection();
     if (selection.isEmpty())
       return;
@@ -217,7 +228,6 @@ void PlaylistView::removeSelected()
 *******************************************************************************/
 void PlaylistView::jumpToCurrentlyPlayingTrack()
 {
-    /* WARNING test */  
     MEDIA::TrackPtr track = Engine::instance()->playingTrack();
     int row = m_model->rowForTrack(track);
     
@@ -250,14 +260,21 @@ void PlaylistView::paintEvent(QPaintEvent *event)
       font.setPointSize(10);
       painter.setFont(font);
 
-      const QRect textBox = rect().adjusted(10, 0, -10, 0);
+      const QRect textBox = rect().adjusted(10, this->height()/2-70, -10, 0);
 
       painter.setPen(QApplication::palette().color(QPalette::Disabled,QPalette::WindowText) );
 
       painter.drawText(textBox.adjusted(0, 70, 0, 0), Qt::AlignHCenter | Qt::TextWordWrap, message);
 
-      const QPixmap pix = QPixmap(":/images/info-message.png") ;
-      painter.drawPixmap(textBox.width()/2 - pix.width()/2, 10 ,pix);
+      QPixmap p_in = QPixmap(":/images/info-48x48.png");
+
+      QPixmap p_out(p_in.size());
+      p_out.fill(Qt::transparent);
+      QPainter p(&p_out);
+      p.setOpacity(0.2);
+      p.drawPixmap(0, 0, p_in);
+      p.end();
+      painter.drawPixmap(textBox.width()/2 - p_out.width()/2, textBox.y() ,p_out);
     }
 
     if (m_drop_indicator_row != -1)
@@ -308,7 +325,6 @@ void PlaylistView::paintEvent(QPaintEvent *event)
 
 void PlaylistView::dragMoveEvent(QDragMoveEvent *event)
 {
-
     QListView::dragMoveEvent(event);
     QModelIndex index(indexAt(event->pos()));
     m_drop_indicator_row = index.isValid() ? index.row() : 0;
@@ -336,10 +352,75 @@ void PlaylistView::dropEvent(QDropEvent *event)
 
 void PlaylistView::mouseMoveEvent(QMouseEvent* event)
 {
-  if (!m_drag_over)
-    QListView::mouseMoveEvent(event);
+    if(ACTIONS()->value(PLAYQUEUE_OPTION_SHOW_RATING)->isChecked())
+    {
+        QModelIndex index = indexAt(event->pos());
+        if(index.isValid() &&  (event->pos().x() >= this->viewport()->width() - 75))
+        {
+            m_delegate->set_mouse_over(index, event->pos() );
+            update(index);
+    
+            foreach (const QModelIndex& index, selectedIndexes())
+              update(index);
+        }
+        else if(m_delegate->is_mouse_over())
+        {
+            m_delegate->set_mouse_out();
+            update(index);
+
+            foreach (const QModelIndex& index, selectedIndexes())
+              update(index);
+        }
+    }
+  
+    if (!m_drag_over)
+      QListView::mouseMoveEvent(event);
 }
 
+void PlaylistView::leaveEvent(QEvent* event) 
+{
+    //Debug::debug() << "    [PlaylistView] leaveEvent";
+    
+    if (m_delegate->is_mouse_over()) {
+
+      const QModelIndex old_index = m_delegate->mouse_over_index();
+      m_delegate->set_mouse_out();
+      update(old_index);
+    }
+
+    QListView::leaveEvent(event);
+}
+
+void PlaylistView::mousePressEvent(QMouseEvent* event) 
+{
+    QModelIndex index = indexAt(event->pos());
+    if (event->button() == Qt::LeftButton && index.isValid() )
+    {
+      if(event->pos().x() >= this->viewport()->width() - 75) 
+      {
+        float hover_rating = RatingPainter::RatingForPos(m_delegate->mouse_over_pos(), QRect(viewport()->width()-75, 0, 80, 16));
+
+        QList<MEDIA::MediaPtr> tracks;
+        if (selectedIndexes().contains(index)) {
+          foreach (const QModelIndex& idx, selectedIndexes()) {
+            MEDIA::TrackPtr track = m_model->trackAt( idx.row() );
+            track->rating = hover_rating;
+            tracks << track; 
+          }
+        }
+        else {
+            MEDIA::TrackPtr track = m_model->trackAt( index.row() );
+            track->rating = hover_rating;
+            tracks << track; 
+        }
+
+        /* rate in database */  
+        QtConcurrent::run(DatabaseCmd::rateMediaItems, tracks);
+      }  
+    }
+  
+    QListView::mousePressEvent(event);
+}
 /*******************************************************************************
     PlaylistView::keyPressEvent
 *******************************************************************************/
@@ -377,7 +458,7 @@ void PlaylistView::keyPressEvent(QKeyEvent* event)
 *******************************************************************************/
 void PlaylistView::slot_lastfm_love()
 {
-    Debug::debug() << "--- PlaylistView : slot_lastfm_love";
+    //Debug::debug() << "    [PlaylistView] slot_lastfm_love";
     QItemSelection selection = selectionModel()->selection();
     if (selection.isEmpty())
       return;
@@ -411,7 +492,6 @@ PlaylistDelegate::PlaylistDelegate(QObject *parent, PlayqueueModel* model)
 {
     m_model      = model;
 
-    icon_media_playing  = QIcon(":/images/media-playing.png");
     icon_media_broken   = QIcon(":/images/media-broken-18x18.png");
     icon_media_track    = QIcon(":/images/track-48x48.png");
     icon_media_stream   = QIcon(":/images/media-url-48x48.png");
@@ -424,118 +504,145 @@ PlaylistDelegate::PlaylistDelegate(QObject *parent, PlayqueueModel* model)
     font_bold   = QApplication::font();
     font_bold.setStyleStrategy(QFont::PreferAntialias);
     font_bold.setBold(true);
-
-    _mode = PLAYQUEUE::MODE_EXTENDED;
 }
 
 void PlaylistDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
 {
-    //! get the data object
+    /* get the data object */
     QModelIndex mappedIndex = m_model->proxy()->mapToSource(index);
     MEDIA::TrackPtr track   = m_model->trackAt(mappedIndex.row());
 
-    bool isTrack     = (track->type() == TYPE_TRACK) ? true : false;
-    bool isBroken    = track->isBroken;
-    bool isSelected  = option.state & QStyle::State_Selected;
+    const bool isTrack     = (track->type() == TYPE_TRACK) ? true : false;
+    const bool isBroken    = track->isBroken;
+    const bool isSelected  = option.state & QStyle::State_Selected;
 
     const int left   = option.rect.left();
     const int top    = option.rect.top();
     const int width  = option.rect.width();
     const int height = option.rect.height();
 
-    // Turn on antialiasing
+    /* Turn on antialiasing*/
     //painter->setRenderHint(QPainter::Antialiasing, true);
   
-    //! draw background
+    /* draw background */
     QStyleOptionViewItemV4 opt(option);
+    opt.state = option.state;
+    opt.showDecorationSelected = true;
+    opt.state &= ~(QStyle::State_Selected | QStyle::State_MouseOver);
+    opt.state |= QStyle::State_Enabled;
     opt.state |= QStyle::State_Active;
     opt.state |= QStyle::State_HasFocus;
+    if( isSelected )
+      opt.state |= QStyle::State_Selected;
+    
+    UTIL::getStyle()->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
 
-    QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
-    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
-
-    //! set painter font & color
+    /* set painter font & color*/
     painter->setFont(font_normal);
     painter->setPen( opt.palette.color ( (track->id!=-1 || isSelected) ? QPalette::Normal : QPalette::Disabled, isSelected ? QPalette::HighlightedText : QPalette::WindowText) );
 
     QFontMetrics fm( font_normal );
 
-    /*---------------------------------------------*/
-    /* TITLE/ALBUM/ARTIST view                     */
-    /* --------------------------------------------*/
-    if(_mode != PLAYQUEUE::MODE_EXTENDED)
+
+    /* Paint Icon */
+    int leftoffset  = 3;
+    int rightoffset = 0;
+
+    if(ACTIONS()->value(PLAYQUEUE_OPTION_SHOW_COVER)->isChecked()) 
     {
-        //! Paint Icon
-        QIcon icon = this->getIcon(track);
-        icon.paint(painter, left + 3, top + 2, 16, 16, Qt::AlignCenter, QIcon::Normal);
+        if(track->isPlaying)
+          UTIL::drawPlayingIcon(painter,height, 17, QPoint(left,top));
+        else
+          this->getIcon(track).paint(painter, leftoffset, top+2, height-4, height-4, Qt::AlignCenter, QIcon::Normal);
 
-        //! Paint duration
-        QRect rectDuree = QRect();
-        if(isTrack && !isBroken)
-        {
-          const QString durationText = track->durationToString();
-          painter->drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);
-          rectDuree = fm.boundingRect ( durationText );
-        }
-
-        //! Paint track info
-        QString trackinfo = this->getTrackInfo(track);
-        trackinfo = fm.elidedText ( trackinfo, Qt::ElideRight, width-51 );
-        painter->drawText(left + 25,top,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, trackinfo);
+        leftoffset += height+4; //icon size = (height-4) + 8 for padding
     }
-    /*---------------------------------------------*/
-    /* EXTENDED view                               */
-    /* --------------------------------------------*/
-    else
+    else 
     {
-        //! Paint Icon
-        QIcon icon = this->getIcon(track);
-        icon.paint(painter, left + 3, top+(height-16)/2, 16, 16, Qt::AlignCenter, QIcon::Normal);
+        if(track->isPlaying)
+          UTIL::drawPlayingIcon(painter,20, 0, QPoint(leftoffset, top+(height-16)/2));
+        else
+          this->getIcon(track).paint(painter, leftoffset, top+(height-16)/2, 16, 16, Qt::AlignCenter, QIcon::Normal);
+        
+        leftoffset += 16+8; //icon size = 16 + 8 for padding
+    }
 
-        // Prepare rectangle for painting
-        QRect rectDuree = QRect();
+    /* Prepare rectangle for painting*/
+    QRect rectDuree = QRect();
 
-        if(!isBroken)
+    if(!isBroken)
+    {
+        if(isTrack)
         {
-            if(isTrack)
+            /* track in collection -> draw painting */
+            if(ACTIONS()->value(PLAYQUEUE_OPTION_SHOW_RATING)->isChecked())
             {
-              //! Paint duration
+              rightoffset = 75;
+              float hover_rating = -1.0;
+              if(track->id != -1)
+              {
+                  if(m_mouse_over_index == index || (
+                     m_view->selectionModel()->selectedIndexes().contains(m_mouse_over_index) &&
+                     m_view->selectionModel()->selectedIndexes().contains(index)))
+                  {
+                      hover_rating = RatingPainter::RatingForPos(m_mouse_over_pos, QRect(m_view->viewport()->width()-75, 0, 80, 16));
+                  }
+              }
+
+              RatingPainter::instance()->Paint(painter, QRect(width-75, top+height/2, 75, 16), hover_rating == -1.0 ? track->rating : hover_rating, true);
+
+              /* paint duration */
               const QString durationText = track->durationToString();
-              painter->drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);
+              painter->drawText(width - 50,top+height/2-fm.height()-1, 49, fm.height()+2, Qt::AlignTop | Qt::AlignRight, durationText);
               rectDuree = fm.boundingRect ( durationText );
             }
-
-            //! Paint track info
-            QString info_2  = isTrack ? track->artist + " - " + track->album : track->url;
-
-            info_2 = fm.elidedText ( info_2, Qt::ElideRight, width-51 );
-            painter->drawText(left + 25,top+height/2+1,width-rectDuree.width()-5, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, info_2);
-
-            painter->setFont(font_bold);
-            fm = QFontMetrics(font_bold);
-
-            QString track_title  = isTrack ? track->title : track->name;
-            track_title = fm.elidedText ( track_title, Qt::ElideRight, width-51 );
-            painter->drawText(left + 25,top+height/2-fm.height()-1,width-rectDuree.width()-5, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, track_title);
+            else
+            {
+              /* paint duration */
+              const QString durationText = track->durationToString();
+              painter->drawText(width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);
+              rectDuree = fm.boundingRect ( durationText );
+              rightoffset = rectDuree.width();
+            }
         }
+
+        /* Paint track info*/
+        QString info_2  = isTrack ? track->artist + " - " + track->album : track->url;
+        info_2 = fm.elidedText ( info_2, Qt::ElideRight, width-leftoffset-rightoffset-5 );
+        painter->drawText(leftoffset,top+height/2+1,width-leftoffset-rightoffset, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, info_2);
+
+        painter->setFont(font_bold);
+        fm = QFontMetrics(font_bold);
+
+
+        QString track_title;
+        if( isTrack && ACTIONS()->value(PLAYQUEUE_OPTION_SHOW_NUMBER)->isChecked() )
+            track_title = UTIL::prettyTrackNumber(track->num ) + " - " + track->title;
+        else if( isTrack )
+            track_title = track->title;
         else
-        {
-            //! Paint track info
-            QString track_info = fm.elidedText ( track->url, Qt::ElideRight, width-51 );
-            painter->drawText(left + 25,top+height/2+1,width-rectDuree.width()-5, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, track_info);
+            track_title = track->extra["station"].toString();
+        
+        track_title = fm.elidedText ( track_title, Qt::ElideRight, width-leftoffset-rectDuree.width()-5 );
+        painter->drawText(leftoffset,top+height/2-fm.height()-1,width-leftoffset-rectDuree.width(), fm.height()+2,Qt::AlignTop | Qt::AlignLeft, track_title);
+    }
+    else
+    {
+        /* Paint track info*/
+        QString track_info = fm.elidedText ( track->url, Qt::ElideRight, width-leftoffset-5 );
+        painter->drawText(leftoffset,top+height/2+1,width-leftoffset, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, track_info);
 
-            painter->setFont(font_bold);
-            fm = QFontMetrics(font_bold);
+        painter->setFont(font_bold);
+        fm = QFontMetrics(font_bold);
 
-            QString info_1  = tr("media reading error");
-            info_1 = fm.elidedText ( info_1, Qt::ElideRight, width-51 );
-            painter->drawText(left + 25,top+height/2-fm.height()-1,width-rectDuree.width()-5, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, info_1);
-        }
+        QString info_1  = tr("media reading error");
+        info_1 = fm.elidedText ( info_1, Qt::ElideRight, width-leftoffset );
+        painter->drawText(leftoffset,top+height/2-fm.height()-1,width-leftoffset-5, fm.height()+2,Qt::AlignTop | Qt::AlignLeft, info_1);
     }
 
-    //! draw flag for "stop after" action
+    /* draw flag for "stop after" action */
     if(track->isStopAfter) {
-      int y = (height - 16)/2 /* 16 = stopbox height */;
+      const int y = (height - 16)/2 /* 16 = stopbox height */;
       drawStop(painter, QRect(left + width - 41,top+y, 40, 16) );
     }
 }
@@ -546,51 +653,36 @@ Q_UNUSED(index)
     int height;
     int min;
     const int padding = 4;
-    if (_mode == PLAYQUEUE::MODE_EXTENDED)
-    {
-      height  =    QFontMetrics(font_normal).height()
+
+    height  =    QFontMetrics(font_normal).height()
                  + QFontMetrics(font_bold).height()
                  + 4 ; /*padding between line */
-      min = cst_minHeightExtended;
-    }
-    else
-    {
-      height = QFontMetrics(font_normal).height();
-      min = cst_minHeightNormal;
-    }
+    min = cst_minHeightExtended;
+
     return  QSize( option.rect.width(), qMax(height + padding * 2, min) );
 }
 
-QString PlaylistDelegate::getTrackInfo(const MEDIA::TrackPtr track) const
-{
-    if(track->isBroken) {
-      return track->url;
-    }
-    else if (track->type() == TYPE_TRACK) {
-        switch( _mode ) {
-          case PLAYQUEUE::MODE_TITLE  : return QString(track->title);break;
-          case PLAYQUEUE::MODE_ALBUM  : return QString(track->album + " - " + track->title);break;
-          case PLAYQUEUE::MODE_ARTIST : return QString(track->artist + " - " + track->album + " - " + track->title);break;
-          default : return track->title;break;
-        }
-    }
-    else {
-      return track->name;
-    }
-
-    return QString::null;
-}
 
 QIcon PlaylistDelegate::getIcon(const MEDIA::TrackPtr track) const
 {
-    if(track->isPlaying)
-      return icon_media_playing;
-    else if (track->isBroken)
+    if (track->isBroken)
+    {
       return icon_media_broken;
-    else if (track->type() == TYPE_TRACK)
-      return icon_media_track;
+    }
     else
-      return icon_media_stream;
+    {
+        if ( !ACTIONS()->value(PLAYQUEUE_OPTION_SHOW_COVER)->isChecked() ) 
+        {  
+            if(track->type() == TYPE_TRACK)
+              return icon_media_track;
+            else
+              return icon_media_stream;
+        }        
+        else
+        {
+            return QIcon( CoverCache::instance()->cover(track) );
+        }
+    }
 }
 
 void PlaylistDelegate::drawStop(QPainter * painter, QRect rect) const
@@ -603,7 +695,7 @@ void PlaylistDelegate::drawStop(QPainter * painter, QRect rect) const
 
     QLinearGradient gradient(rect.topLeft(), rect.bottomLeft());
     gradient.setColorAt(0.0,  qRgb(102, 150, 227) );
-    gradient.setColorAt(1.0, qRgb(77, 121, 200));
+    gradient.setColorAt(1.0,  qRgb(77, 121, 200));
 
     painter->setRenderHint(QPainter::Antialiasing);
 
@@ -615,3 +707,4 @@ void PlaylistDelegate::drawStop(QPainter * painter, QRect rect) const
     // Draw the text
     painter->drawText(rect, Qt::AlignCenter, "stop");
 }
+
