@@ -1,6 +1,6 @@
 /****************************************************************************************
 *  YAROCK                                                                               *
-*  Copyright (c) 2010-2014 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
+*  Copyright (c) 2010-2016 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
 *                                                                                       *
 *  This program is free software; you can redistribute it and/or modify it under        *
 *  the terms of the GNU General Public License as published by the Free Software        *
@@ -16,14 +16,14 @@
 *****************************************************************************************/
 
 #include "browser_view.h"
-#include "bottom_container.h"
 
+/* models */
 #include "models/local/local_track_model.h"
 #include "models/local/local_playlist_model.h"
 #include "models/local/histo_model.h"
 #include "models/stream/stream_model.h"
 
-// scene
+/* scene */
 #include "views/local/local_scene.h"
 #include "views/stream/stream_scene.h"
 #include "views/context/context_scene.h"
@@ -31,19 +31,22 @@
 #include "views/settings/settings_scene.h"
 #include "views/about/about_scene.h"
 
-// core
+/* core */
 #include "threadmanager.h"
 #include "engine.h"
+#include "media_search.h"
 
-// widget 
+/* widgets */
 #include "widgets/main/menumodel.h"
-#include "widgets/main/centraltoolbar.h"
-#include "widgets/statuswidget.h"
+#include "widgets/main/main_left.h"
+#include "widgets/statusmanager.h"
 
 #include "settings.h"
 #include "global_actions.h"
 #include "debug.h"
 
+#include <QScrollBar>
+#include <QApplication>
 /*
 ********************************************************************************
 *                                                                              *
@@ -53,11 +56,11 @@
 */
 BrowserView::BrowserView(QWidget *parent) : QGraphicsView(parent)
 {
-    //! widget setup
+    /* widget setup */
     this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
     this->setAutoFillBackground(false);
 
-    //! GraphicView Setup
+    /* GraphicView Setup */
     this->setFrameShape(QFrame::NoFrame);
     this->setFrameShadow(QFrame::Plain);
 
@@ -84,28 +87,52 @@ BrowserView::BrowserView(QWidget *parent) : QGraphicsView(parent)
     m_scrollbar->installEventFilter(this);
     m_scrollbar->setMinimum( 0 );
 
-    /* file scene */
-    FileScene* file_scene = new FileScene(this);
-    m_scenes.insert(VIEW::ViewFileSystem,  file_scene);
-    connect(file_scene, SIGNAL(load_directory(const QString&)), this, SLOT(slot_on_load_new_data(const QString&)));
+    /* Build scene */  
+      /* filesystem scene */    
+      m_scenes.insert(VIEW::ViewFileSystem,  new FileScene(this));
+      connect(static_cast<FileScene*>(m_scenes[VIEW::ViewFileSystem]), SIGNAL(load_directory(QVariant)), this, SLOT(slot_on_load_new_data(QVariant)));
 
-    /* about scene */    
-    AboutScene* about_scene = new AboutScene(this);
-    m_scenes.insert(VIEW::ViewAbout,  about_scene);
+      /* about scene */    
+      m_scenes.insert(VIEW::ViewAbout,  new AboutScene(this));
     
-    
-    /* initialization        */  
-    m_browser_params_idx = -1;
+      /* settings */
+      m_scenes.insert(VIEW::ViewSettings,  new SettingsScene(this));
+      connect(static_cast<SettingsScene*>(m_scenes[VIEW::ViewSettings]), SIGNAL(settings_saved()), this, SIGNAL(settings_saved()));
 
+      /* context */
+      m_scenes.insert(VIEW::ViewContext,  new ContextScene(this) );      
+ 
+      /* stream scene */
+      m_scenes.insert(VIEW::ViewDirble,         new StreamScene(this) );
+      m_scenes.insert(VIEW::ViewRadionomy,      m_scenes[VIEW::ViewDirble] );
+      m_scenes.insert(VIEW::ViewTuneIn,         m_scenes[VIEW::ViewDirble] );
+      m_scenes.insert(VIEW::ViewFavoriteRadio,  m_scenes[VIEW::ViewDirble] );
+
+      connect(static_cast<StreamScene*>(m_scenes[VIEW::ViewDirble]), SIGNAL(linked_changed(QVariant)), this, SLOT(slot_on_load_new_data(QVariant)));
+      
+      /* local scene */
+      m_scenes.insert(VIEW::ViewArtist,         new LocalScene(this) );
+      m_scenes.insert(VIEW::ViewAlbum,          m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewTrack,          m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewGenre,          m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewYear,           m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewDashBoard,      m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewHistory,        m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewPlaylist,       m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewSmartPlaylist,  m_scenes[VIEW::ViewArtist] );
+      m_scenes.insert(VIEW::ViewFavorite,       m_scenes[VIEW::ViewArtist] );
+      connect(static_cast<LocalScene*>(m_scenes[VIEW::ViewArtist]), SIGNAL(linked_changed(QVariant)), this, SLOT(slot_on_load_new_data(QVariant)));
+    
     /* connections */
-    connect(CentralToolBar::instance(), SIGNAL(explorerFilterActivated(const QString&)),this, SLOT(slot_on_search_changed(const QString&)));
+    connect(MainLeftWidget::instance(), SIGNAL(browser_search_change(const QVariant&)),this, SLOT(slot_on_search_changed(const QVariant&)));
     connect(ThreadManager::instance(),  SIGNAL(modelPopulationFinished(E_MODEL_TYPE)), this, SLOT(slot_on_model_populated(E_MODEL_TYPE)));
     
     
     connect(ACTIONS()->value(BROWSER_PREV), SIGNAL(triggered()), this, SLOT(slot_on_history_prev_activated()));
     connect(ACTIONS()->value(BROWSER_NEXT), SIGNAL(triggered()), this, SLOT(slot_on_history_next_activated()));
 
-    connect(MenuModel::instance(),SIGNAL(modelItemActivated(QModelIndex)),this, SLOT(slot_on_menu_index_changed(QModelIndex)));    
+    connect(MenuModel::instance(),SIGNAL(menu_browser_triggered(VIEW::Id, QVariant)),this, SLOT(slot_on_menu_browser_triggered(VIEW::Id, QVariant)));
+    connect(ACTIONS()->value(TAG_CLICKED), SIGNAL(triggered()), this, SLOT(slot_on_tag_clicked()));
     
     connect (m_scrollbar, SIGNAL(actionTriggered(int)), this, SLOT(slot_check_slider(int)));
       
@@ -114,61 +141,38 @@ BrowserView::BrowserView(QWidget *parent) : QGraphicsView(parent)
     connect(ACTIONS()->value(BROWSER_JUMP_TO_TRACK), SIGNAL(triggered()), SLOT(slot_jump_to_media()));
     connect(ACTIONS()->value(BROWSER_JUMP_TO_MEDIA), SIGNAL(triggered()), SLOT(slot_jump_to_media()));
     
-    /* connections */
-    setViewportMargins(0,0,0,0);
-    
-    /* bottom widget container */  
-    m_bottomWidget = new BottomContainer();
-
-    this->viewport()->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding );
-    
-    QGridLayout* gridLayout = new QGridLayout();
-    gridLayout->setSpacing(0);
-    gridLayout->setMargin(0);
-    gridLayout->addWidget(this->viewport(),0,0);
-    gridLayout->addWidget(m_bottomWidget,1,0);
-
-    this->setLayout(gridLayout);    
-
-    show_bottom_panel(0);
-    
+    /* initialization        */  
+    m_browser_params_idx = -1;    
     m_menu = 0;
 }
 
 /*******************************************************************************
-    set scene method
+   playSelectedItems
 *******************************************************************************/
-void BrowserView::setLocalScene(LocalScene *scene)
+bool BrowserView::playSelectedItems()
 {
-    /* register local scene */
-    m_scenes.insert(VIEW::ViewArtist,         scene );
-    m_scenes.insert(VIEW::ViewAlbum,          scene );
-    m_scenes.insert(VIEW::ViewTrack,          scene );
-    m_scenes.insert(VIEW::ViewGenre,          scene );
-    m_scenes.insert(VIEW::ViewYear,           scene );
-    m_scenes.insert(VIEW::ViewDashBoard,      scene );
-    m_scenes.insert(VIEW::ViewHistory,        scene );
-    m_scenes.insert(VIEW::ViewPlaylist,       scene );
-    m_scenes.insert(VIEW::ViewSmartPlaylist,  scene );
-    m_scenes.insert(VIEW::ViewFavorite,       scene );
+    LocalScene* localscene = static_cast<LocalScene*>(m_scenes[VIEW::ViewArtist]);
+    StreamScene* streamscene = static_cast<StreamScene*>(m_scenes[VIEW::ViewDirble]);
+  
+    if(!localscene->selectedItems().isEmpty()) {
+      localscene->playSelected();
+      return true;
+    }
+        
+    if (!streamscene->selectedItems().isEmpty()) {
+      streamscene->playSelected();
+      return true;
+    }  
+
+    return false;
 }
 
-void BrowserView::setStreamScene(StreamScene *scene)
+/*******************************************************************************
+   settingsResults
+*******************************************************************************/
+SETTINGS::Results BrowserView::settingsResults()
 {
-    m_scenes.insert(VIEW::ViewDirble,         scene );
-    m_scenes.insert(VIEW::ViewShoutCast,      scene );
-    m_scenes.insert(VIEW::ViewTuneIn,         scene );
-    m_scenes.insert(VIEW::ViewFavoriteRadio,  scene );    
-}
-
-void BrowserView::setContextScene(ContextScene *scene)
-{
-    m_scenes.insert(VIEW::ViewContext,  scene );      
-}
-
-void BrowserView::setSettingsScene(SettingsScene *scene)
-{
-    m_scenes.insert(VIEW::ViewSettings, scene );      
+    return static_cast<SettingsScene*>(m_scenes[VIEW::ViewSettings])->results();
 }
 
 /*******************************************************************************
@@ -180,8 +184,9 @@ void BrowserView::restore_view()
   
    BrowserParam param;
 
-   param.mode   = VIEW::Id(SETTINGS()->_viewMode);
-   param.filter = QString();
+   param.mode    = VIEW::Id(SETTINGS()->_viewMode);
+   param.scroll  = SETTINGS()->_browserScroll;
+   param.search  = QVariant();
    
    if(param.mode == VIEW::ViewFileSystem)
      param.data   = QVariant( SETTINGS()->_filesystem_path );
@@ -195,14 +200,20 @@ void BrowserView::restore_view()
    is_started = true;
 }
 
+void BrowserView::save_view()
+{
+    Debug::debug() << "  [BrowserView] save_view  ";
+  
+    SETTINGS()->_browserScroll = m_scrollbar->sliderPosition();
+}
 
 /*******************************************************************************
    Changing view slots 
 *******************************************************************************/
 /* slot triggered when use enter the search field */
-void BrowserView::slot_on_search_changed(const QString& filter)
+void BrowserView::slot_on_search_changed(const QVariant& variant)
 {           
-    Debug::debug() << "  [BrowserView] slot_on_search_changed  " << filter;
+    Debug::debug() << "  [BrowserView] slot_on_search_changed  ";
 
     BrowserParam current_param;
     if(m_browser_params_idx != -1)
@@ -211,51 +222,97 @@ void BrowserView::slot_on_search_changed(const QString& filter)
   
     BrowserParam param = BrowserParam(
            VIEW::Id(SETTINGS()->_viewMode),
-           filter,
+           variant,
            current_param.data);
     
     add_history_entry(param);
 
     switch_view(param);
+
+    update_statuswidget();
 }
 
 /* slot triggered when action from menu is activated */
-void BrowserView::slot_on_menu_index_changed(QModelIndex idx)
+void BrowserView::slot_on_menu_browser_triggered(VIEW::Id view, QVariant data)
 {
-    if(!idx.isValid()) return;
-   
-    VIEW::Id m_mode    = VIEW::Id (idx.data(ViewModeRole).toInt());
-    QVariant       m_data    = idx.data(FileRole);   
-    QString        m_filter  = CentralToolBar::instance()->explorerFilter();
+    Debug::debug() << "  [BrowserView] slot_on_menu_browser_triggered data";
+  
+    QVariant search = MainLeftWidget::instance()->browserSearch();
 
-    BrowserParam param = BrowserParam(m_mode,m_filter,m_data);
+    /* manage search case */
+    if( (typeForView(view) != VIEW::LOCAL) || (view == VIEW::ViewSmartPlaylist) )
+    {
+      /* reset the advance search if the new mode can not support advance search */
+      if(search.canConvert<MediaSearch>())
+        search = QVariant();
+    }
+       
+    BrowserParam param = BrowserParam(view,search,data);
     
     add_history_entry(param);
 
     switch_view(param);
+    
+    update_statuswidget();
 }
 
-/* slot used by FileScene to load a directory inside view */
-void BrowserView::slot_on_load_new_data(const QString& data)
+/* slot slot_on_load_new_data                              */
+/*    -> used by FileScene to load a directory inside view */
+/*    -> used by StreamScene on new link                   */
+void BrowserView::slot_on_load_new_data(const QVariant data)
 {
-    Debug::debug() << "  [BrowserView] slot_on_load_new_data data" << data;
+    Debug::debug() << "  [BrowserView] slot_on_load_new_data";
 
+    MEDIA::LinkPtr link = qvariant_cast<MEDIA::LinkPtr>(data);
+    
     BrowserParam param = BrowserParam(
           VIEW::Id(SETTINGS()->_viewMode),
-          CentralToolBar::instance()->explorerFilter(),
-          QVariant( data ));
+          MainLeftWidget::instance()->browserSearch(),
+          data);
     
-    add_history_entry( param );
-       
+    /* hack because stream service notify every state change */
+    /* no need to record all state, i.e DOWNLOADING state    */
+    if( data.canConvert<QString>() || (qvariant_cast<MEDIA::LinkPtr>(data))->state == SERVICE::DATA_OK )
+      add_history_entry( param );
+            
     switch_view(param);
+    
+    if( data.canConvert<QString>() || (qvariant_cast<MEDIA::LinkPtr>(data))->state == SERVICE::DATA_OK )
+      update_statuswidget();
 }
 
+
+void BrowserView::slot_on_tag_clicked()
+{
+    Debug::debug() << "  [BrowserView] slot_on_tag_clicked";
+    BrowserParam current_param;
+    if(m_browser_params_idx != -1)
+      current_param = m_browser_params.at(m_browser_params_idx);
+
+  
+    BrowserParam param = BrowserParam(
+           VIEW::ViewGenre,
+           ACTIONS()->value(TAG_CLICKED)->data(),
+           QVariant());
+    
+    add_history_entry(param);
+
+    switch_view(param);
+
+    update_statuswidget();
+    
+}
+
+
 void BrowserView::active_view(VIEW::Id m, QString f, QVariant d)
 {
     BrowserParam param = BrowserParam(m, f, d);
 
     add_history_entry( param );
+    
     switch_view(param);
+    
+    update_statuswidget();
 }
 
     
@@ -291,6 +348,8 @@ void BrowserView::slot_on_model_populated(E_MODEL_TYPE model)
       break;
       default : break;
     }
+    
+    update_statuswidget();
 }
 
 /*******************************************************************************
@@ -305,7 +364,7 @@ void BrowserView::resizeEvent( QResizeEvent * event)
     if( scene->isInit() )
       scene->resizeScene();
      
-     event->accept();
+    event->accept();
 }
 
 /*******************************************************************************
@@ -337,19 +396,20 @@ bool BrowserView::eventFilter(QObject *obj, QEvent *ev)
     return QWidget::eventFilter(obj, ev);
 }
 
+    
+    
 /*******************************************************************************
     switch_view
 *******************************************************************************/
 void BrowserView::switch_view(BrowserParam& param)
 {
-    Debug::debug() << "  [BrowserView] switch_view param.mode " << param.mode;
+    //Debug::debug() << "  [BrowserView] switch_view param.mode " << param.mode;
     //Debug::debug() << "  [BrowserView] switch_view param.filter" << param.filter;
     //Debug::debug() << "  [BrowserView] switch_view param.data" << param.data;
+    Debug::debug() << "  [BrowserView] switch_view";
 
     /* save current state */
     SETTINGS()->_viewMode = param.mode;
-
-    CentralToolBar::instance()->showHideGoUp(param.mode);
     
     /* handle scene switch view */
     SceneBase * scene = m_scenes[ param.mode ];
@@ -358,17 +418,17 @@ void BrowserView::switch_view(BrowserParam& param)
       scene->initScene();
           
     scene->setMode( param.mode );
-    scene->setFilter( param.filter );
+    scene->setSearch( param.search );
     scene->setData( param.data );
     scene->populateScene();
+    scene->playSceneContents( param.search );
     this->setFocus();
-    
-    show_bottom_panel( m_scenes[param.mode]->bottomWidget() );
 
-    CentralToolBar::instance()->setExplorerFilterText(param.filter);
-    
+    MainLeftWidget::instance()->setMode(param.mode);
+    MainLeftWidget::instance()->setBrowserSearch(param.search);
 
-    switch( VIEW::typeForView(VIEW::Id(param.mode)) ) {
+    switch( VIEW::typeForView(VIEW::Id(param.mode)) )
+    {
       case VIEW::LOCAL      : setScene(static_cast<LocalScene*>(m_scenes[param.mode]));    break;
       case VIEW::RADIO      : setScene(static_cast<StreamScene*>(m_scenes[param.mode]));   break;
       case VIEW::CONTEXT    : setScene(static_cast<ContextScene*>(m_scenes[param.mode]));  break;
@@ -380,8 +440,8 @@ void BrowserView::switch_view(BrowserParam& param)
     /* restore scroll position */
     m_scrollbar->setSliderPosition(param.scroll);
 
-    /* status widget update */
-    do_statuswidget_update();    
+    /* update page title */
+    MainLeftWidget::instance()->setTitle( name_for_view(VIEW::Id(param.mode)) );
 }
 
     
@@ -437,10 +497,11 @@ void BrowserView::jump_to_media(MEDIA::MediaPtr media)
       mode = VIEW::ViewTrack;
     else return;  
       
-    CentralToolBar::instance()->setExplorerFilterText("");
-    m_scenes[VIEW::ViewArtist]->setFilter("");
-    m_scenes[VIEW::ViewTuneIn]->setFilter("");
-    m_scenes[VIEW::ViewFileSystem]->setFilter("");
+    MainLeftWidget::instance()->setBrowserSearch(QVariant());
+    
+    m_scenes[VIEW::ViewArtist]->setSearch(QVariant());
+    m_scenes[VIEW::ViewTuneIn]->setSearch(QVariant());
+    m_scenes[VIEW::ViewFileSystem]->setSearch(QVariant());
       
     active_view(mode,"", QVariant());
 
@@ -448,7 +509,7 @@ void BrowserView::jump_to_media(MEDIA::MediaPtr media)
     QPoint p = static_cast<LocalScene*>(m_scenes[mode])->get_item_position(media);
 
     if(!p.isNull()) 
-      this->verticalScrollBar()->setSliderPosition( p.y() - 40 );
+      m_scrollbar->setSliderPosition( p.y() - 40 );
 }
 
 
@@ -516,28 +577,6 @@ void BrowserView::slot_on_history_next_activated()
 }
 
 
-/*******************************************************************************
-    show/hide bottom panel
-*******************************************************************************/
-void BrowserView::show_bottom_panel(QWidget* w)
-{
-    //Debug::debug() << "  [BrowserView] show_bottom_panel";
-
-    m_bottomWidget->setContent( w );
-   
-    if(!w) 
-    {
-      m_bottomWidget->hide();
-      setViewportMargins(0,0,0,0);
-    }
-    else
-    {
-      m_bottomWidget->show();  
-
-      m_bottomWidget->resize(m_bottomWidget->width(),w->sizeHint().height() + 4);
-      setViewportMargins(0,0,0,w->sizeHint().height() + 4);
-    }
-}
 
 /*******************************************************************************
     contextMenuEvent
@@ -553,8 +592,14 @@ void BrowserView::contextMenuEvent ( QContextMenuEvent * event )
     }
     else 
     {
-       if(!m_menu)
+       if(!m_menu) 
+       {
          m_menu = new QMenu(this);
+         
+         m_menu->setStyleSheet(
+             QString("QMenu {icon-size: 32px; background-color: none;border: none;}"
+                     "QMenu::item {background-color: none;}") );         
+       }
        
        m_menu->clear();
        m_menu->addActions(scene->actions());
@@ -602,11 +647,11 @@ void BrowserView::keyPressEvent ( QKeyEvent * event )
 }
 
 /*******************************************************************************
-    updateStatusWidget
+    update_statuswidget
 *******************************************************************************/
-void BrowserView::do_statuswidget_update()
+void BrowserView::update_statuswidget()
 {
-    //Debug::debug() << "  [BrowserView] do_statuswidget_update";
+    //Debug::debug() << "  [BrowserView] update_statuswidget";
 
     const int collectionSize = static_cast<LocalScene*>(m_scenes[VIEW::ViewArtist])->elementCount();
     const int radioSize      = static_cast<StreamScene*>(m_scenes[VIEW::ViewTuneIn])->elementCount();
@@ -615,8 +660,6 @@ void BrowserView::do_statuswidget_update()
 
     switch( SETTINGS()->_viewMode )
     {
-      case (VIEW::ViewContext)  : text = tr("Context");break;
-      case (VIEW::ViewHistory)  : text = tr("History");break;
       case (VIEW::ViewAlbum)    : text = QString(tr("Collection : <b>%1</b> albums")).arg(QString::number(collectionSize)); break;
       case (VIEW::ViewArtist)   : text = QString(tr("Collection : <b>%1</b> artist")).arg(QString::number(collectionSize));break;
       case (VIEW::ViewTrack)    : text = QString(tr("Collection : <b>%1</b> tracks")).arg(QString::number(collectionSize));break;
@@ -630,7 +673,7 @@ void BrowserView::do_statuswidget_update()
         break;
       
       case (VIEW::ViewDirble)         :
-      case (VIEW::ViewShoutCast)      :
+      case (VIEW::ViewRadionomy)      :
       case (VIEW::ViewTuneIn)         :
       case (VIEW::ViewFavoriteRadio)  :
          text = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(radioSize));
@@ -638,8 +681,42 @@ void BrowserView::do_statuswidget_update()
       default: text = "";break;
     }
 
-    if(!text.isEmpty())
-      StatusWidget::instance()->startShortMessage(text, STATUS::TYPE_INFO, 2500);
+    if( (VIEW::typeForView(VIEW::Id(SETTINGS()->_viewMode)) == VIEW::LOCAL && collectionSize != 0) ||
+        (VIEW::typeForView(VIEW::Id(SETTINGS()->_viewMode)) == VIEW::RADIO && radioSize != 0) )
+    {
+        StatusManager::instance()->startMessage(text, STATUS::INFO, 2200);
+    }
+}
+
+/*******************************************************************************
+    name_for_view
+*******************************************************************************/
+QString BrowserView::name_for_view(VIEW::Id id)
+{
+    QString title;
+    switch( id )
+    {
+      case VIEW::ViewAbout             : title = tr("About");  break;
+      case VIEW::ViewSettings          : title = tr("Settings");  break;
+      case VIEW::ViewDashBoard         : title = tr("Dashboard");  break;
+      case VIEW::ViewContext           : title = tr("Context");  break;
+      case VIEW::ViewHistory           : title = tr("History");  break;
+      case VIEW::ViewArtist            : title = tr("Artists");  break;
+      case VIEW::ViewAlbum             : title = tr("Albums");  break;
+      case VIEW::ViewTrack             : title = tr("Tracks");  break;
+      case VIEW::ViewGenre             : title = tr("Genres");  break;
+      case VIEW::ViewYear              : title = tr("Years");  break;
+      case VIEW::ViewFavorite          : title = tr("Favorites");  break;
+      case VIEW::ViewPlaylist          : title = tr("Playlists");  break;
+      case VIEW::ViewSmartPlaylist     : title = tr("Smart playlists");  break;
+      case VIEW::ViewDirble            : title = ("Dirble");  break;
+      case VIEW::ViewRadionomy         : title = ("Radionomy");  break;
+      case VIEW::ViewTuneIn            : title = ("TuneIn");  break;
+      case VIEW::ViewFavoriteRadio     : title = tr("Favorites radios");  break;
+      case VIEW::ViewFileSystem        : title = tr("Filesystem");  break;
+      default:break;
+    }     
+    return title;
 }
 
 
@@ -660,7 +737,7 @@ void BrowserView::slot_check_slider(int action)
 
       if( categories.isEmpty() ) return;
         
-      int top = this->verticalScrollBar()->sliderPosition();
+      int top = m_scrollbar->sliderPosition();
                   
       /* ------------ navigate go up ---------------- */
       if( action == QAbstractSlider::SliderPageStepSub) 
@@ -670,7 +747,7 @@ void BrowserView::slot_check_slider(int action)
             QPoint itemPoint = categories.at(i)->scenePos().toPoint();
     
             if(top > itemPoint.y()) {
-              this->verticalScrollBar()->setSliderPosition(itemPoint.y());
+              m_scrollbar->setSliderPosition(itemPoint.y());
               break;
             }
           }
@@ -683,7 +760,7 @@ void BrowserView::slot_check_slider(int action)
             QPoint itemPoint = (item->scenePos()).toPoint();
 
             if(top < itemPoint.y()) {
-              this->verticalScrollBar()->setSliderPosition(itemPoint.y());
+              m_scrollbar->setSliderPosition(itemPoint.y());
               break;
             }
           }      
