@@ -1,6 +1,6 @@
 /****************************************************************************************
 *  YAROCK                                                                               *
-*  Copyright (c) 2010-2014 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
+*  Copyright (c) 2010-2016 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
 *                                                                                       *
 *  This program is free software; you can redistribute it and/or modify it under        *
 *  the terms of the GNU General Public License as published by the Free Software        *
@@ -25,6 +25,8 @@
 #include "playqueue/virtual_playqueue.h"
 
 #include "core/player/engine.h"
+#include "covercache.h"
+
 
 #include "mediaitem.h"
 #include "global_actions.h"
@@ -32,9 +34,16 @@
 
 #include "debug.h"
 
-#include <QtGui>
-
+#include <QApplication>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QToolBar>
+#include <QWidgetAction>
+#include <QLabel>
 
+#include <QGraphicsView>
+#include <QGraphicsSceneEvent>
+#include <QGraphicsScene>
 
 /*
 ********************************************************************************
@@ -48,54 +57,38 @@ GraphicsItemMenu::GraphicsItemMenu(QWidget *parent) : QMenu(parent)
     this->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
     this->setAutoFillBackground(true);
     this->setContentsMargins(0,0,0,0);
-    this->setWindowFlags( Qt::Popup | Qt::FramelessWindowHint );
-    
-    QPalette palette = QApplication::palette();
-    palette.setColor(QPalette::Background, palette.color(QPalette::Base));
-    this->setPalette(palette);
- 
+
+    this->setStyleSheet( "QMenu {background-color: none;border: none;}");
+         
     /* menu actions */
     m_map_actions.insert(ALBUM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(ALBUM_LOAD_COVER,    new QAction(QIcon(":/images/album.png"), tr("Load cover art from file"), 0));
-    m_map_actions.insert(ALBUM_LASTFM_COVER,  new QAction(QIcon(":/images/download-48x48.png"), tr("Download cover art"), 0));
-    m_map_actions.insert(ALBUM_REMOVE_COVER,  new QAction(QIcon::fromTheme("edit-delete"), tr("Remove cover art"), 0));
-    m_map_actions.insert(ALBUM_EDIT,          new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
-    m_map_actions.insert(ALBUM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(ALBUM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
-    m_map_actions.insert(ALBUM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
+    m_map_actions.insert(ALBUM_QUEUE_END,     new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
+    
+    m_map_actions.insert(UPDATE_FAVORITE,     new QAction(QIcon(":/images/favorites-48x48.png"), "", 0));
+    m_map_actions.insert(EDIT,                new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
 
     m_map_actions.insert(GENRE_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(GENRE_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(GENRE_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
-    m_map_actions.insert(GENRE_EDIT,          new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
+    m_map_actions.insert(GENRE_QUEUE_END,     new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
 
     m_map_actions.insert(ARTIST_PLAY,         new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(ARTIST_EDIT,         new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
-    m_map_actions.insert(ARTIST_QUEUE_END,    new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(ARTIST_QUEUE_NEW,    new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
-    m_map_actions.insert(ARTIST_FAVORITE,     new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
+    m_map_actions.insert(ARTIST_QUEUE_END,    new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
 
     m_map_actions.insert(PLAYLIST_PLAY,       new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(PLAYLIST_EDIT,       new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
-    m_map_actions.insert(PLAYLIST_QUEUE_END,  new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(PLAYLIST_QUEUE_NEW,  new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
-    m_map_actions.insert(PLAYLIST_FAVORITE,   new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
+    m_map_actions.insert(PLAYLIST_QUEUE_END,  new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
     m_map_actions.insert(PLAYLIST_REMOVE,     new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove playlist from disk"), 0));
 
     m_map_actions.insert(TRACK_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(TRACK_EDIT,          new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
-    m_map_actions.insert(TRACK_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(TRACK_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
+    m_map_actions.insert(TRACK_QUEUE_END,     new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
 
     m_map_actions.insert(STREAM_PLAY,          new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
     m_map_actions.insert(STREAM_EDIT,          new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
-    m_map_actions.insert(STREAM_QUEUE_END,     new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(STREAM_QUEUE_NEW,     new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
+    m_map_actions.insert(STREAM_QUEUE_END,     new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
     m_map_actions.insert(STREAM_FAVORITE,      new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add to favorites"), 0));
-
+    m_map_actions.insert(STREAM_WEBSITE,       new QAction(QIcon(":/images/media-url-48x48.png"), tr("Website"), 0));
+    
     m_map_actions.insert(SELECTION_PLAY,       new QAction(QIcon(":/images/media-play.png"), tr("Play"), 0));
-    m_map_actions.insert(SELECTION_QUEUE_END,  new QAction(QIcon(":/images/playqueue_enqueue_end-48x48.png"), tr("Enqueue to playqueue"), 0));
-    m_map_actions.insert(SELECTION_QUEUE_NEW,  new QAction(QIcon(":/images/playqueue_enqueue_new-48x48.png"), tr("Enqueue as new playqueue"), 0));
+    m_map_actions.insert(SELECTION_QUEUE_END,  new QAction(QIcon(":/images/media-playlist-48x48.png"), tr("Add to play queue"), 0));
+    m_map_actions.insert(SELECTION_TRACK_EDIT, new QAction(QIcon(":/images/edit-48x48.png"), tr("Edit"), 0));
 
 
     /* signals connections */
@@ -125,13 +118,12 @@ void GraphicsItemMenu::appendItems(QList<QGraphicsItem*> items)
 
 void GraphicsItemMenu::updateMenu(bool isSelection)
 {
-    Debug::debug() << "GraphicsItemMenu::updateMenu";
+    //Debug::debug() << "GraphicsItemMenu::updateMenu";
   
     /* delete action_widget */
     this->clear();
     m_scene = 0;
     
-    
   
     is_selection = isSelection;
   
@@ -139,9 +131,16 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
 
 
     /* main widget*/
-    QWidget* main_widget = new QWidget();
+    QWidget* main_widget = new QWidget(this);
     main_widget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
-
+             
+    QLabel* ui_label = new QLabel(this);
+    ui_label->setFont(QFont("Arial",14,QFont::Normal));
+    ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
+    ui_label->setStyleSheet(
+        QString("QLabel { color : %1; }").arg(QColor(SETTINGS()->_baseColor).name())
+    );
+    
     /*-------------------------------------------------*/
     /* Selected items                                  */
     /* ------------------------------------------------*/
@@ -158,15 +157,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
         default: break;
       }    
     
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
-
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-    
       ui_label->setText(text);
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
       
 
       /* tool bar content */
@@ -174,8 +165,10 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(SELECTION_PLAY) );
       tool_bar->addAction( m_map_actions.value(SELECTION_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(SELECTION_QUEUE_NEW) );
 
+      if ( m_items.first()->type() == GraphicsItem::TrackType )
+        tool_bar->addAction( m_map_actions.value(SELECTION_TRACK_EDIT) );
+      
       QHBoxLayout * hbox = new QHBoxLayout();
       hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
       hbox->addWidget(tool_bar);
@@ -184,7 +177,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(2);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addLayout(hbox);
       main_widget->setLayout(main_box);
@@ -196,7 +189,8 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
     {
       m_scene = new QGraphicsScene(main_widget);
       m_view = new QGraphicsView(main_widget);
-    
+      m_view->setStyleSheet("background: transparent");
+      
       /* QGraphicsView setup */  
       m_view->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
       m_view->setAutoFillBackground(false);
@@ -246,30 +240,17 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
     
       m_scene->setSceneRect ( m_scene->itemsBoundingRect().adjusted(0, 0, 10, 10) );
 
-
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
-
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-    
       ui_label->setText(album->name);
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
     
       /* tool bar content */
       QToolBar *tool_bar = new QToolBar(main_widget);
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(ALBUM_PLAY) );
       tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_QUEUE_NEW) );
       tool_bar->addSeparator();
-      tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_EDIT) );
+      tool_bar->addAction( m_map_actions.value(EDIT) );
       tool_bar->addSeparator();
-      tool_bar->addAction( m_map_actions.value(ALBUM_FAVORITE) );
+      tool_bar->addAction( m_map_actions.value(UPDATE_FAVORITE) );
       tool_bar->addSeparator();
       
       tool_bar->addAction( ACTIONS()->value(BROWSER_JUMP_TO_MEDIA) );
@@ -285,14 +266,14 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
       if(item->media->isFavorite)
-        m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Remove from favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Remove from favorites") );
       else
-        m_map_actions.value(ALBUM_FAVORITE)->setText( tr("Add to favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Add to favorites") );
 
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addWidget(m_view);
       main_box->addLayout(hbox);
@@ -305,6 +286,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
     {
       m_scene = new QGraphicsScene(main_widget);
       m_view = new QGraphicsView(main_widget);
+      m_view->setStyleSheet("background: transparent");
     
       /* QGraphicsView setup */  
       m_view->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
@@ -353,15 +335,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       }    
       m_scene->setSceneRect ( m_scene->itemsBoundingRect().adjusted(0, 0, 10, 10) );
     
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
-
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-    
       ui_label->setText(album->name);
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);      
       
       
       /* tool bar content */
@@ -369,12 +343,8 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(GENRE_PLAY) );
       tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(GENRE_QUEUE_NEW) );
       tool_bar->addSeparator();
-      tool_bar->addAction( m_map_actions.value(ALBUM_LOAD_COVER) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_LASTFM_COVER) );
-      tool_bar->addAction( m_map_actions.value(ALBUM_REMOVE_COVER) );
-      tool_bar->addAction( m_map_actions.value(GENRE_EDIT) );
+      tool_bar->addAction( m_map_actions.value(EDIT) );
       tool_bar->addSeparator();
 
       tool_bar->addAction( ACTIONS()->value(BROWSER_JUMP_TO_MEDIA) );
@@ -392,7 +362,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addWidget(m_view);
       main_box->addLayout(hbox);
@@ -406,6 +376,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       //Debug::debug() << "GraphicsItemMenu::ArtistType";
       m_scene = new QGraphicsScene(main_widget);
       m_view = new QGraphicsView(main_widget);
+      m_view->setStyleSheet("background: transparent");
     
       /* QGraphicsView setup */  
       m_view->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
@@ -458,27 +429,17 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       
       m_scene->setSceneRect ( m_scene->itemsBoundingRect().adjusted(0, 0, 10, 10) );
       
-    
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
-
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);    
       ui_label->setText(artist->name);
-       
 
       /* tool bar content */
       QToolBar *tool_bar = new QToolBar(main_widget);
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(ARTIST_PLAY) );
       tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(ARTIST_QUEUE_NEW) );
       tool_bar->addSeparator();
-      tool_bar->addAction( m_map_actions.value(ARTIST_EDIT) );
+      tool_bar->addAction( m_map_actions.value(EDIT) );
       tool_bar->addSeparator();
-      tool_bar->addAction( m_map_actions.value(ARTIST_FAVORITE) );
+      tool_bar->addAction( m_map_actions.value(UPDATE_FAVORITE) );
       tool_bar->addSeparator();
 
       tool_bar->addAction( ACTIONS()->value(BROWSER_JUMP_TO_MEDIA) );
@@ -494,14 +455,14 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
 
 
       if(item->media->isFavorite)
-        m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Remove from favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Remove from favorites") );
       else
-        m_map_actions.value(ARTIST_FAVORITE)->setText( tr("Add to favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Add to favorites") );
 
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addWidget(m_view);
       main_box->addLayout(hbox);
@@ -516,19 +477,12 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       TrackGraphicItem *item = static_cast<TrackGraphicItem*>(m_items.first());
       MEDIA::TrackPtr track = MEDIA::TrackPtr::staticCast(item->media);
 
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
-
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
       
       /* hack for stream in playlist item */
       if( MEDIA::isLocal(item->media->url) )
-        ui_label->setText(track->title);      
+        ui_label->setText( track->title );
       else
-        ui_label->setText(track->name); /* for stream */
+        ui_label->setText( track->extra["station"].toString() );
       
       /* tool bar content */
       QToolBar *tool_bar = new QToolBar(main_widget);
@@ -536,10 +490,9 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       tool_bar->addAction( m_map_actions.value(TRACK_PLAY) );
 
       if(item->media->id != -1)
-        tool_bar->addAction( m_map_actions.value(TRACK_EDIT) );
+        tool_bar->addAction( m_map_actions.value(EDIT) );
 
       tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(TRACK_QUEUE_NEW) );
 
       QHBoxLayout * hbox = new QHBoxLayout();
       hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
@@ -549,7 +502,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addLayout(hbox);
       main_widget->setLayout(main_box);
@@ -563,29 +516,22 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(m_items.first());
       MEDIA::PlaylistPtr playlist = MEDIA::PlaylistPtr::staticCast(item->media);
       
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
       
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);    
       ui_label->setText(playlist->name);
       
       QToolBar *tool_bar = new QToolBar(main_widget);
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(PLAYLIST_PLAY) );
       tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(PLAYLIST_QUEUE_NEW) );
       tool_bar->addSeparator();
       tool_bar->addAction( m_map_actions.value(PLAYLIST_REMOVE) );
-      tool_bar->addAction( m_map_actions.value(PLAYLIST_EDIT) );
-      tool_bar->addAction( m_map_actions.value(PLAYLIST_FAVORITE) );
+      tool_bar->addAction( m_map_actions.value(EDIT) );
+      tool_bar->addAction( m_map_actions.value(UPDATE_FAVORITE) );
 
       if(item->media->isFavorite)
-        m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Remove from favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Remove from favorites") );
       else
-        m_map_actions.value(PLAYLIST_FAVORITE)->setText( tr("Add to favorites") );
+        m_map_actions.value(UPDATE_FAVORITE)->setText( tr("Add to favorites") );
 
       QHBoxLayout * hbox = new QHBoxLayout();
       hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
@@ -595,7 +541,7 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       /* main layout */
       QVBoxLayout * main_box = new QVBoxLayout();
       main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
+      main_box->setContentsMargins(4,4,4,4);
       main_box->addWidget(ui_label);
       main_box->addLayout(hbox);
       main_widget->setLayout(main_box);
@@ -609,26 +555,25 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       StreamGraphicItem *item = static_cast<StreamGraphicItem*>(m_items.first());
       MEDIA::TrackPtr stream = MEDIA::TrackPtr::staticCast(item->media);
 
-      QPalette palette;
-      palette.setColor(QPalette::WindowText, SETTINGS()->_baseColor);
       
-      QLabel* ui_label = new QLabel(this);
-      ui_label->setFont(QFont("Arial",14,QFont::Normal));
-      ui_label->setPalette(palette);    
-      ui_label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);    
-      ui_label->setText(stream->name);
+      ui_label->setText( stream->extra["station"].toString() );
+      
+      QLabel* ui_pix = new QLabel(this);
+      ui_pix->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);    
+      ui_pix->setPixmap( CoverCache::instance()->cover(stream).scaled(QSize(48,48), Qt::KeepAspectRatio, Qt::SmoothTransformation) );
+      
       
       
       QToolBar *tool_bar = new QToolBar(main_widget);
       tool_bar->setIconSize( QSize( 16, 16 ) );
       tool_bar->addAction( m_map_actions.value(STREAM_PLAY) );
       tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_END) );
-      tool_bar->addAction( m_map_actions.value(STREAM_QUEUE_NEW) );
       tool_bar->addAction( m_map_actions.value(STREAM_FAVORITE) );
 
       if(m_view_mode == VIEW::ViewFavoriteRadio)
         tool_bar->addAction( m_map_actions.value(STREAM_EDIT) );
 
+      tool_bar->addAction( m_map_actions.value(STREAM_WEBSITE) );
       
       if(item->media->isFavorite)
         m_map_actions.value(STREAM_FAVORITE)->setText( tr("Remove from favorites") );
@@ -641,18 +586,23 @@ void GraphicsItemMenu::updateMenu(bool isSelection)
       hbox->addWidget(tool_bar);
       hbox->addItem(new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
 
+      
+      QVBoxLayout * vbox = new QVBoxLayout();
+      vbox->setSpacing(0);
+      vbox->setContentsMargins(4,4,4,4);
+      vbox->addWidget(ui_label);
+      vbox->addLayout(hbox);
+      
       /* main layout */
-      QVBoxLayout * main_box = new QVBoxLayout();
-      main_box->setSpacing(0);
-      main_box->setContentsMargins(0,0,0,0);
-      main_box->addWidget(ui_label);
-      main_box->addLayout(hbox);
-      main_widget->setLayout(main_box);
+      QHBoxLayout * main_layout = new QHBoxLayout();
+      main_layout->addWidget( ui_pix );
+      main_layout->addLayout( vbox );
+      main_widget->setLayout(main_layout);
     }
 
     QWidgetAction* action_widget = new QWidgetAction(this);
     action_widget->setDefaultWidget( main_widget );
-    this->addAction( action_widget );  
+    this->addAction( action_widget );
 }
 
 
@@ -673,7 +623,7 @@ void GraphicsItemMenu::slot_actionsTriggered()
 *******************************************************************************/
 bool GraphicsItemMenu::eventFilter(QObject *obj, QEvent *ev)
 {
-    //Debug::debug() << "BrowserView eventFilter  obj" << obj;
+    //Debug::debug() << "GraphicsItemMenu eventFilter  obj" << obj;
 
     if (obj == this)
     {
@@ -683,8 +633,7 @@ bool GraphicsItemMenu::eventFilter(QObject *obj, QEvent *ev)
 
     if (wid && (wid == m_scene) && (ev->type() == QEvent::GraphicsSceneMouseDoubleClick) )
     {
-        //Debug::debug() << "###### qgraphics view event GraphicsSceneMouseDoubleClick";
-        QGraphicsItem* item = m_scene->itemAt(static_cast<QGraphicsSceneMouseEvent*>(ev)->scenePos());
+        QGraphicsItem* item = m_scene->itemAt(static_cast<QGraphicsSceneMouseEvent*>(ev)->scenePos(), QTransform());
 
         if(item) {
           if(item->type() == GraphicsItem::TrackType)
@@ -712,7 +661,8 @@ bool GraphicsItemMenu::eventFilter(QObject *obj, QEvent *ev)
 *******************************************************************************/
 void GraphicsItemMenu::slot_updateScene()
 {
-Debug::debug() << "GraphicsItemMenu::slot_updateScene";
-  if(m_scene)
-    m_scene->update();
+    //Debug::debug() << "GraphicsItemMenu::slot_updateScene";
+    if(m_scene)
+      m_scene->update();
 }
+
