@@ -1,6 +1,6 @@
 /****************************************************************************************
 *  YAROCK                                                                               *
-*  Copyright (c) 2010-2014 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
+*  Copyright (c) 2010-2016 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
 *                                                                                       *
 *  This program is free software; you can redistribute it and/or modify it under        *
 *  the terms of the GNU General Public License as published by the Free Software        *
@@ -26,9 +26,9 @@
 
 #include "models/stream/stream_model.h"
 #include "models/stream/service_dirble.h"
-#include "models/stream/service_shoutcast.h"
+#include "models/stream/service_favorite.h"
+#include "models/stream/service_radionomy.h"
 #include "models/stream/service_tunein.h"
-#include "models/stream/service_xspf.h"
 
 #include "playqueue/virtual_playqueue.h"
 #include "playqueue/playqueue_model.h"
@@ -43,17 +43,19 @@
 #include "widgets/exlineedit.h"
 #include "widgets/dialogs/addstreamdialog.h"
 #include "dialogs/filedialog.h"
-#include "widgets/statuswidget.h"
+#include "widgets/statusmanager.h"
 #include "views/item_button.h"
 
 #include "global_actions.h"
 #include "settings.h"
 #include "debug.h"
 
-#include <QtGui>
+#include <QGraphicsView>
 #include <QGraphicsProxyWidget>
 #include <QGraphicsWidget>
-
+#include <QDrag>
+#include <QColor>
+#include <QtGui/QDesktopServices>
 /*
 ********************************************************************************
 *                                                                              *
@@ -82,12 +84,13 @@ void StreamScene::initScene()
 
     /* init stream services */
     m_services.insert(VIEW::ViewDirble,         new Dirble() );
-    m_services.insert(VIEW::ViewShoutCast,      new ShoutCast() );
+    m_services.insert(VIEW::ViewRadionomy,      new Radionomy() );
     m_services.insert(VIEW::ViewTuneIn,         new TuneIn() );
-    m_services.insert(VIEW::ViewFavoriteRadio,  new XspfStreams() );
-
+    m_services.insert(VIEW::ViewFavoriteRadio,  new FavoriteStreams() );
+    
+    
     foreach (Service* service, m_services.values() ) {
-      connect(service, SIGNAL(stateChanged()), this, SLOT(populateScene()));      
+      connect(service, SIGNAL(stateChanged()), this, SLOT(slot_on_service_state_changed()));
       connect(service, SIGNAL(dataChanged()), this, SLOT(update()));
     }    
    
@@ -99,53 +102,35 @@ void StreamScene::initScene()
     m_mouseGrabbedItem    = 0;
     m_infoSize            = 0;
 
-    /*  scene actions */
-    m_actions.insert("add_favorite", new QAction(QIcon(":/images/add_32x32.png"), tr("add"),this));
-    m_actions.insert("import_favorite", new QAction(QIcon(":/images/add_32x32.png"), tr("import"),this));
-
-    m_actions.insert("reload", new QAction(QIcon(":/images/rebuild.png"),QString("reload"),this));
-    connect(m_actions.value("reload"), SIGNAL(triggered()), this, SLOT(slot_reload_stream_list()));
-    connect(m_actions.value("add_favorite"), SIGNAL(triggered()), this, SLOT(slot_on_add_stream_clicked()));
-    connect(m_actions.value("import_favorite"), SIGNAL(triggered()), this, SLOT(slot_on_import_stream_clicked()));
-
     /* drag items action */
     ACTIONS()->insert(BROWSER_STREAM_ITEM_MOUSE_MOVE, new QAction(this));
     
     connect(ACTIONS()->value(BROWSER_STREAM_ITEM_MOUSE_MOVE), SIGNAL(triggered()), this, SLOT(slot_item_mouseMove()), Qt::DirectConnection);
  
-    /* search line edit */
-    ui_search_widget = new QWidget(0);
-    ui_search_widget->setPalette(QApplication::palette());
-    ui_search_widget->setAttribute(Qt::WA_NoBackground, true);
-    ui_search_widget->setAutoFillBackground(true);
-//     ui_search_widget->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
-
-    QLabel* label = new QLabel(ui_search_widget);
-    label->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
-    label->setText("search");
-
-    ui_line_edit = new QLineEdit(ui_search_widget);
-    ui_line_edit->setMinimumWidth(120);
-    ui_line_edit->setMaximumWidth(120);
-    ui_line_edit->setMinimumHeight(26);
-    ui_line_edit->setMaximumHeight(26);
-
-    QHBoxLayout* layout = new QHBoxLayout(ui_search_widget);
-    layout->setContentsMargins(0, 0, 0, 0);
-    layout->setSpacing(0);
-    layout->setMargin(0);    
-    layout->addWidget(label);
-    layout->addWidget(ui_line_edit);
-    layout->addItem(new QSpacerItem(60, 20, QSizePolicy::Minimum, QSizePolicy::Minimum));
-
-    ui_proxy = this->addWidget(ui_search_widget, Qt::Widget);
-//     ui_proxy->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
-
-    connect(ui_line_edit, SIGNAL(returnPressed()), this, SLOT(slot_on_search_activated()), Qt::QueuedConnection);
+    /* ---- search line edit ---- */
+    ui_ex_line_edit = new ExLineEdit(0);
+    ui_ex_line_edit->setInactiveText(tr("Search"));
     
-    ui_search_widget->hide();
+    QPalette p = QApplication::palette();
+    QColor c = SETTINGS()->_baseColor ;
+    c.setAlphaF( 0.6 );
     
-    /* end init */    
+    p.setColor( QPalette::Background, c );
+    p.setColor( QPalette::Highlight, c );
+    p.setColor( QPalette::Base, c );
+    ui_ex_line_edit->setPalette( p );
+   
+    ui_ex_line_edit->setStyleSheet( "QWidget, QWidget:focus { border: none; }" );
+    
+
+    ui_proxy = this->addWidget(ui_ex_line_edit);
+    ui_proxy->setFlag(QGraphicsItem::ItemIsFocusable); 
+
+    connect(ui_ex_line_edit, SIGNAL(textfield_entered()),this, SLOT(slot_on_search_activated()));
+
+    ui_ex_line_edit->hide();
+    
+    /* ---- end init ---- */
     setInit(true);
 }
     
@@ -155,6 +140,24 @@ void StreamScene::initScene()
 *******************************************************************************/
 QList<QAction *> StreamScene::actions() 
 {
+    /*actions are requested on BrowserView::contextMenuEvent */
+    if(m_actions.isEmpty())
+    {
+        /*  scene actions */
+        m_actions.insert("add_favorite", new QAction(QIcon(":/images/add_32x32.png"), tr("add"),this));
+        m_actions.insert("import_favorite", new QAction(QIcon(":/images/add_32x32.png"), tr("import"),this));
+        m_actions.insert("reload", new QAction(QIcon(":/images/rebuild.png"),QString("reload"),this));
+
+        connect(m_actions.value("reload"), SIGNAL(triggered()), this, SLOT(slot_reload_stream_list()));
+        connect(m_actions.value("add_favorite"), SIGNAL(triggered()), this, SLOT(slot_on_add_stream_clicked()));
+        connect(m_actions.value("import_favorite"), SIGNAL(triggered()), this, SLOT(slot_on_import_stream_clicked()));
+    }
+    
+    /* update actions */
+    m_actions.value("add_favorite")->setEnabled( m_services[mode()]->type() == SERVICE::LOCAL);
+    m_actions.value("import_favorite")->setEnabled( m_services[mode()]->type() == SERVICE::LOCAL);
+
+    
     QList<QAction*> list;
     list << m_actions.value("add_favorite") 
          << m_actions.value("import_favorite")
@@ -164,20 +167,50 @@ QList<QAction *> StreamScene::actions()
 }
 
 /*******************************************************************************
-     setFilter
+     setSearch
 *******************************************************************************/
-void StreamScene::setFilter(const QString& filter)
+void StreamScene::setSearch(const QVariant& variant)
 {
-    m_model->setFilter(filter);
+    m_model->setFilter( variant.toString() );
 }
 
+/*******************************************************************************
+     setData
+*******************************************************************************/
+void StreamScene::setData(const QVariant& data)
+{
+    MEDIA::LinkPtr active_link = qvariant_cast<MEDIA::LinkPtr>(data);
+    
+    if( active_link ) {
+      Debug::debug() << "StreamScene::setData active link" << active_link->name;
+        
+      m_services[mode()]->setActiveLink(active_link);
+    }
+}
+
+
 /*******************************************************************************
      resizeScene
 *******************************************************************************/
 void StreamScene::resizeScene()
 {
-    //Debug::debug() << "   [StreamScene] resizeScene";   
-    update();
+    //Debug::debug() << "   [LocalScene] resizeScene";   
+    int new_item_count = (parentView()->width()/160 > 2) ? parentView()->width()/160 : 2;
+
+    if(item_count != new_item_count)  
+    {
+      populateScene();
+    }
+    else
+    {
+      if(mode() != VIEW::ViewFavoriteRadio) 
+      {
+        QWidget *w = qobject_cast<QGraphicsView*> (parentView())->viewport();
+        ui_proxy->setPos(w->width() - 13 - ui_proxy->widget()->width(), 2);
+      }
+    
+      update();
+    }
 }
 
 /*******************************************************************************
@@ -188,15 +221,12 @@ void StreamScene::populateScene()
     Debug::debug() << "   [StreamScene] PopulateScene";
 
     /* clear the scene FIRST */
-    ui_search_widget->hide();    
-    if(ui_proxy->scene())
+    ui_ex_line_edit->hide();    
+    if( ui_proxy->scene() )
       removeItem(ui_proxy);
+
     clear();
     
-    /* update actions */
-    m_actions.value("add_favorite")->setEnabled( m_services[mode()]->type() == SERVICE::LOCAL);
-    m_actions.value("import_favorite")->setEnabled( m_services[mode()]->type() == SERVICE::LOCAL);
-    
     /* handle service state */
     SERVICE::State state = m_services[mode()]->state();
         
@@ -236,28 +266,37 @@ void StreamScene::populateExtendedStreamScene()
 {
     Debug::debug() << "   [StreamScene] populateExtendedStreamScene";
   
-    int streamRow      = 0;
-    int categoriesRow  = 0;
     m_infoSize         = 0;
     int Xpos = 20,Ypos = 5;
-
-
-    /*  header item */
-    HeaderItem* header = new HeaderItem(28, qobject_cast<QGraphicsView*> (parentView())->viewport());
-    header->setText( m_services[mode()]->name() );
-    header->setPos(0, Ypos);
-   
-    addItem(header);
+    int Column    = 0;
+    item_count = (parentView()->width()/160 > 2) ? parentView()->width()/160 : 2;
     
-    Ypos += 55;
+    const int categoriesHeight = 40;
    
     /*  search field */    
-    if(mode() != VIEW::ViewFavoriteRadio) {
+    if(mode() != VIEW::ViewFavoriteRadio)
+    {
       addItem(ui_proxy); 
      
-      ui_proxy->setPos(4, 6);
-      ui_search_widget->show();
-      ui_line_edit->setText( m_services[mode()]->searchTerm() );
+      QWidget *w = qobject_cast<QGraphicsView*> (parentView())->viewport();
+      ui_proxy->setPos(w->width() - 13 - ui_proxy->widget()->width(), 2);
+      ui_ex_line_edit->show();
+      ui_ex_line_edit->setText( m_services[mode()]->searchTerm() );
+    }
+    else
+    {
+      ButtonStateItem* button = new ButtonStateItem();
+      button->setText(m_services[mode()]->rootLink()->name);
+      button->setChecked( m_services[mode()]->activeLink() == m_services[mode()]->rootLink() );
+      QVariant v;
+      v.setValue(static_cast<MEDIA::LinkPtr>(m_services[mode()]->rootLink()));
+      button->setData(v);
+  
+      connect(button, SIGNAL(clicked()), m_services[mode()], SLOT(slot_activate_link()));
+      
+      button->setPos(Xpos ,Ypos );
+      Xpos = Xpos + button->width() + 20;
+      addItem(button);    
     }
     
     /*--------------------------------------------------*/
@@ -266,26 +305,31 @@ void StreamScene::populateExtendedStreamScene()
     QList<MEDIA::LinkPtr> links = m_services[mode()]->links();
     QString current_category;
     
+    if( mode() == VIEW::ViewFavoriteRadio)
+      current_category = m_services[mode()]->rootLink()->name;
+ 
     foreach(MEDIA::LinkPtr link, links) 
     {
-      if(current_category != link->categorie)
+      if( link->parent() && current_category != MEDIA::LinkPtr::staticCast(link->parent())->name)
       {        
         if(Xpos > 20 ) 
           Ypos = Ypos + 50;
         
-        //! new category
-        current_category = link->categorie;
+        /*  new category */
+        current_category = MEDIA::LinkPtr::staticCast(link->parent())->name;
+        
         CategorieGraphicItem *category = new CategorieGraphicItem(qobject_cast<QGraphicsView*> (parentView())->viewport());
-        category->m_name = link->categorie;
-        category->setPos(0 , Ypos + categoriesRow*40);
+        category->m_name = current_category;
+        category->setPos(0 , Ypos );
 
         addItem(category);
-        categoriesRow++;
+        Ypos += categoriesHeight;
         Xpos = 20;
       }
 
-      ButtonItem* button = new ButtonItem();
+      ButtonStateItem* button = new ButtonStateItem();
       button->setText(link->name);
+      button->setChecked( link == m_services[mode()]->activeLink() );
       QVariant v;
       v.setValue(static_cast<MEDIA::LinkPtr>(link));
       button->setData(v);
@@ -297,65 +341,85 @@ void StreamScene::populateExtendedStreamScene()
         Ypos = Ypos + 30;
       }
 
-      button->setPos(Xpos ,Ypos + categoriesRow*40);
+      button->setPos(Xpos ,Ypos );
       Xpos = Xpos + button->width() + 20;
 
       addItem(button);
-    } // end foreach link
+    } /* end foreach link */
      
-    if(Xpos > 20 ) 
+    if(Xpos != 0 ) 
+    {
       Ypos = Ypos + 50;
-     
+      Xpos = 0;
+    }
+    
     /*--------------------------------------------------*/
     /* stream list                                      */
     /* -------------------------------------------------*/
-    //! stream model loop
+    /* stream model loop */
     for ( int i = 0; i < m_model->itemCount(); i++ )
     {
       if(!m_model->isStreamFiltered(i)) continue;
 
-      if(current_category != m_model->streamAt(i)->categorie)
+      if(current_category != m_model->streamAt(i)->genre)
       {        
-        //! new category
-        current_category = m_model->streamAt(i)->categorie;
+        if(Xpos != 0) {
+          Ypos +=170;
+          Xpos = 0;
+          Column=0;
+        }
+          
+        /* new category */
+        current_category = m_model->streamAt(i)->genre;
         CategorieGraphicItem *category = new CategorieGraphicItem(qobject_cast<QGraphicsView*> (parentView())->viewport());
         category->m_name = current_category;
-        category->setPos(0 , Ypos + categoriesRow*40);
+        category->setPos(0 , Ypos );
 
         addItem(category);
-        categoriesRow++;
+        Ypos += categoriesHeight;
       }      
 
-      if(mode() == VIEW::ViewTuneIn) {
+      m_infoSize++;
+      
+      StreamGraphicItem *stream_item = new StreamGraphicItem();
+      stream_item->media = m_model->streamAt(i);
+      stream_item->media->isFavorite = (mode() == VIEW::ViewFavoriteRadio);
+      stream_item->setPos(4+Xpos, Ypos);
+      addItem( stream_item );
 
-        StreamGraphicItem_v2 *stream_item = new StreamGraphicItem_v2();
-        stream_item->media = m_model->streamAt(i);
-        stream_item->media->isFavorite = (mode() == VIEW::ViewFavoriteRadio);
-        stream_item->setPos(30, Ypos + categoriesRow*40 + streamRow*80);
-        stream_item->_width   = parentView()->width()-30-20;
-        stream_item->setToolTip(stream_item->media->url);
-
-        addItem(stream_item);
+      if(Column < (item_count-1)) {
+        Column++;
+        Xpos+=160;
       }
       else {
-        StreamGraphicItem *stream_item = new StreamGraphicItem();
-        stream_item->media = m_model->streamAt(i);
-        stream_item->media->isFavorite = (mode() == VIEW::ViewFavoriteRadio);
-        stream_item->setPos(30, Ypos + categoriesRow*40 + streamRow*stream_item->height());
-        stream_item->_width   = parentView()->width()-30-20;
-        stream_item->setToolTip(stream_item->media->url);
-
-        addItem(stream_item);
+        Column = 0;
+        Ypos +=170;
+        Xpos=0;
       }
-
-      streamRow++;
-      m_infoSize++;
     }
+
+    if(Xpos != 0)
+        Ypos +=170;
     
+    if(m_services[mode()]->hasMoreLink())
+    {
+      ButtonItem* button = new ButtonItem();
+      button->setText("more");
+      QVariant v;
+      v.setValue(static_cast<MEDIA::LinkPtr>(m_services[mode()]->moreLink()));
+      button->setData(v);
+  
+      connect(button, SIGNAL(clicked()), m_services[mode()], SLOT(slot_activate_link()));      
+      
+      button->setPos(40 ,Ypos );
+      addItem(button);      
+    }
+        
+        
     if( (m_infoSize == 0 && m_model->itemCount()  != 0) || (this->items().count()  <= 1) ) {
         InfoGraphicItem *info = new InfoGraphicItem(qobject_cast<QGraphicsView*> (parentView())->viewport());
         info->_text = "no information";
-        info->setPos( 0 , Ypos + categoriesRow*40 + streamRow*20);
+        info->setPos( 0, Ypos );
         addItem(info);
     }
 }
@@ -364,12 +428,6 @@ void StreamScene::populateExtendedStreamScene()
 
 void StreamScene::populateWaitingScene(QString text)
 {
-    /*  header item */
-    HeaderItem* header = new HeaderItem(qobject_cast<QGraphicsView*> (parentView())->viewport());
-    header->setText( m_services[mode()]->name() );
-    header->setPos(0, 5);
-        
-    addItem(header);
           
     if(m_services[mode()]->state() == SERVICE::DOWNLOADING) { 
       LoadingGraphicItem *info = new LoadingGraphicItem(qobject_cast<QGraphicsView*> (parentView())->viewport());
@@ -398,12 +456,10 @@ void StreamScene::playStream()
     /*--------------------------------------------------*/
     /* urls simple                                      */
     /* -------------------------------------------------*/
-    if( !MEDIA::isPlaylistFile(streamUrl) &&
-        !MEDIA::isShoutCastUrl(streamUrl) &&
-        !MEDIA::isTuneInUrl(streamUrl) )
+    if( MEDIA::isMediaPlayable(streamUrl) )
     {
       Debug::debug() << "   [StreamScene] playStream  we can play it's an url"<< streamUrl;
-
+        
       VirtualPlayqueue::instance()->addTrackAndPlay(item->media);
       return;
     }
@@ -412,7 +468,7 @@ void StreamScene::playStream()
     /* playlist remote or shoutcast                     */
     /* -------------------------------------------------*/
     Debug::debug() << "   [StreamScene] playStream  it's a playlist -> download to do" ;
-    uint i = StatusWidget::instance()->startProgressMessage(tr("Loading stream"));
+    uint i = StatusManager::instance()->startMessage(tr("Loading stream"));
     m_messageIds.insert("StreamLoad", i);
     
     StreamLoader* streamloader = new StreamLoader(item->media);
@@ -425,7 +481,7 @@ void StreamScene::slot_streams_fetched(MEDIA::TrackPtr track)
     QList<MEDIA::TrackPtr> tracks;
     foreach(MEDIA::MediaPtr media, track->children()) {
       tracks << MEDIA::TrackPtr::staticCast(media);
-      //Debug::debug() << "   [StreamScene] slot_streams_fetched URL=" << MEDIA::TrackPtr::staticCast(media)->url;
+      Debug::debug() << "   [StreamScene] slot_streams_fetched URL=" << MEDIA::TrackPtr::staticCast(media)->url;
     }
     
     if(!tracks.isEmpty())
@@ -435,7 +491,7 @@ void StreamScene::slot_streams_fetched(MEDIA::TrackPtr track)
     if(loader) delete loader;
     
     if (m_messageIds.contains("StreamLoad"))
-      StatusWidget::instance()->stopProgressMessage( m_messageIds.take("StreamLoad") );    
+      StatusManager::instance()->stopMessage( m_messageIds.take("StreamLoad") );    
 }
 
 /*******************************************************************************
@@ -462,9 +518,7 @@ void StreamScene::playSelected()
     /*--------------------------------------------------*/
     /* urls simple                                      */
     /* -------------------------------------------------*/
-    if( !MEDIA::isPlaylistFile(streamUrl) &&
-        !MEDIA::isShoutCastUrl(streamUrl) &&
-        !MEDIA::isTuneInUrl(streamUrl) )
+    if( MEDIA::isMediaPlayable(streamUrl) )
     {
         VirtualPlayqueue::instance()->addTrackAndPlay(item->media);
         return;
@@ -475,7 +529,7 @@ void StreamScene::playSelected()
     /* -------------------------------------------------*/
     Debug::debug() << "   [StreamScene] playStream  it's a playlist -> download to do" ;
 
-    uint i = StatusWidget::instance()->startProgressMessage(tr("Loading stream"));
+    uint i = StatusManager::instance()->startMessage(tr("Loading stream"));
     m_messageIds.insert("StreamLoad", i);
     
     StreamLoader* streamloader = new StreamLoader(item->media);
@@ -516,6 +570,7 @@ void StreamScene::enqueueSelected(bool is_new_playqueue)
 *******************************************************************************/
 void StreamScene::enqueueStream(bool is_new_playqueue)
 {
+    Debug::debug() << "StreamScene::enqueueStream";
     if(!m_mouseGrabbedItem) return;
 
     if(is_new_playqueue)
@@ -528,19 +583,22 @@ void StreamScene::enqueueStream(bool is_new_playqueue)
     /*--------------------------------------------------*/
     /* urls simple                                      */
     /* -------------------------------------------------*/
-    if( !MEDIA::isPlaylistFile(streamUrl) &&
-        !MEDIA::isShoutCastUrl(streamUrl) &&
-        !MEDIA::isTuneInUrl(streamUrl) )
+    if( MEDIA::isMediaPlayable(streamUrl) )
     {
       Playqueue::instance()->addMediaItem(item->media);
-      return;
     }
-
     /*--------------------------------------------------*/
     /* playlist remote or shoutcast                     */
     /* -------------------------------------------------*/
-    // use playlist task manager to load streams into playqueue    
-    Playqueue::instance()->manager()->playlistAddFile(streamUrl);
+    else
+    {
+      // use playlist task manager to load streams into playqueue    
+      QList<MEDIA::TrackPtr> tracks; 
+      tracks << item->media;
+      
+      //Playqueue::instance()->manager()->playlistAddFile(streamUrl);
+      Playqueue::instance()->manager()->playlistAddMediaItems(tracks);
+    }
 }
 
 /*******************************************************************************
@@ -616,7 +674,7 @@ QList<QGraphicsItem*> StreamScene::sortedSelectedItem()
 void StreamScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 {
     //Debug::debug() << "   [StreamScene] contextMenuEvent";
-    m_mouseGrabbedItem = this->itemAt(event->scenePos());
+    m_mouseGrabbedItem = this->itemAt(event->scenePos(), QTransform());
 
     if(!m_mouseGrabbedItem) {
       m_graphic_item_menu->hide();
@@ -658,14 +716,24 @@ void StreamScene::slot_contextmenu_triggered(ENUM_ACTION_ITEM_MENU id)
     switch(id)
     {
       case STREAM_PLAY         : playStream();            break;
-      case STREAM_EDIT         : editStream(); break;
+      case STREAM_EDIT         : editStream();            break;
       case STREAM_QUEUE_END    : enqueueStream(false);    break;
-      case STREAM_QUEUE_NEW    : enqueueStream(true);     break;
       case STREAM_FAVORITE     : updateStreamFavorite();  break;
+      case STREAM_WEBSITE      :
+      {
+          if(m_mouseGrabbedItem) 
+          {
+              StreamGraphicItem *item = static_cast<StreamGraphicItem*>(m_mouseGrabbedItem);
+              
+              if( !item->media->extra["website"].toString().isEmpty() )
+                QDesktopServices::openUrl( QUrl(item->media->extra["website"].toString()) );
+              
+              break;   
+          }
+      }
 
       case SELECTION_PLAY      : playSelected();break;
       case SELECTION_QUEUE_END : enqueueSelected(false);break;
-      case SELECTION_QUEUE_NEW : enqueueSelected(true);break;
       default:break;
     }
 }
@@ -692,13 +760,31 @@ void StreamScene::slot_item_mouseMove()
 }
 
 
+/*******************************************************************************
+    mousePressEvent
+*******************************************************************************/
+void StreamScene::mousePressEvent ( QGraphicsSceneMouseEvent * event )
+{
+    //Debug::debug() << "   [StreamScene] mousePressEvent";  
+    m_mouseGrabbedItem = this->itemAt(event->scenePos(), QTransform());
+
+    if( !m_mouseGrabbedItem ) {
+        ui_ex_line_edit->clearFocus();
+        ui_proxy->clearFocus();
+        setFocusItem(0);
+    }
+
+    QGraphicsScene::mousePressEvent(event);
+}
+
+
 /*******************************************************************************
     mouseDoubleClickEvent
 *******************************************************************************/
 void StreamScene::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event )
 {
     //Debug::debug() << "   [StreamScene] mouseDoubleClickEvent";
-    m_mouseGrabbedItem = this->itemAt(event->scenePos());
+    m_mouseGrabbedItem = this->itemAt(event->scenePos(), QTransform());
 
     if(!m_mouseGrabbedItem || m_mouseGrabbedItem->type() != GraphicsItem::StreamType) {
       event->ignore();
@@ -726,16 +812,16 @@ void StreamScene::keyPressEvent ( QKeyEvent * keyEvent )
 
     if((mode() == VIEW::ViewFavoriteRadio) && (keyEvent->key() == Qt::Key_Delete))
     {
-       XspfStreams* xspfstream = static_cast<XspfStreams*>(m_services[VIEW::ViewFavoriteRadio]);
+       FavoriteStreams* favMngr = static_cast<FavoriteStreams*>(m_services[VIEW::ViewFavoriteRadio]);
       
        foreach(QGraphicsItem* gitem, selectedItems())
        {
           StreamGraphicItem *item = static_cast<StreamGraphicItem*>(gitem);
-          xspfstream->updateItem(item->media);
+          favMngr->updateItem(item->media);
        }
 
-       xspfstream->saveToFile();
-       xspfstream->reload();
+       favMngr->saveToDatabase();
+       favMngr->reload();
        
        keyEvent->accept();
        return;
@@ -767,7 +853,7 @@ void StreamScene::slot_on_search_activated()
 {
     Debug::debug() << "   [StreamScene] slot_on_search_activated";
     
-    QLineEdit* line_edit = qobject_cast<QLineEdit*>(sender());
+    ExLineEdit* line_edit = qobject_cast<ExLineEdit*>(sender());
     
     if (!line_edit) return;  
 
@@ -776,11 +862,26 @@ void StreamScene::slot_on_search_activated()
     m_services[mode()]->setSearchTerm( line_edit->text() );
 
     if(!term.isEmpty()) 
-        m_services[mode()]->slot_activate_link( m_services[mode()]->searchLink() );
+      m_services[mode()]->slot_activate_link( m_services[mode()]->searchLink() );
     else
       m_services[mode()]->slot_activate_link( m_services[mode()]->rootLink() );
 }
 
+/*******************************************************************************
+    slot_on_service_state_changed
+*******************************************************************************/
+void StreamScene::slot_on_service_state_changed()
+{
+    Debug::debug() << "   [StreamScene] slot_on_service_state_changed";
+
+    MEDIA::LinkPtr link = m_services[mode()]->activeLink();
+    
+    QVariant v;
+    v.setValue( static_cast<MEDIA::LinkPtr>(link) );
+       
+    emit linked_changed( v );
+}
+
 
 /*******************************************************************************
     Manage favorite
@@ -792,10 +893,10 @@ void StreamScene::updateStreamFavorite()
     if(!m_mouseGrabbedItem) return;
     StreamGraphicItem *item = static_cast<StreamGraphicItem*>(m_mouseGrabbedItem);
 
-    XspfStreams* xspfstream = static_cast<XspfStreams*>(m_services[VIEW::ViewFavoriteRadio]);
-    xspfstream->updateItem(item->media);
-    xspfstream->saveToFile();
-    xspfstream->reload();
+    FavoriteStreams* favMngr = static_cast<FavoriteStreams*>(m_services[VIEW::ViewFavoriteRadio]);
+    favMngr->updateItem(item->media);
+    favMngr->saveToDatabase();
+    favMngr->reload();
 }
 
 void StreamScene::slot_on_add_stream_clicked()
@@ -814,23 +915,24 @@ void StreamScene::slot_on_add_stream_clicked()
       {
           MEDIA::TrackPtr media = MEDIA::TrackPtr(new MEDIA::Track());
           media->setType(TYPE_STREAM);
-          media->id          = -1;
-          media->url         = url;
-          media->name        = !name.isEmpty() ? name : url ;
-          media->categorie   = category;
-          media->isFavorite  = false;
-          media->isPlaying   = false;
-          media->isBroken    = false;
-          media->isPlayed    = false;
-          media->isStopAfter = false;
+          media->id               = -1;
+          media->url              = url;
+          media->extra["station"] = !name.isEmpty() ? name : url ;
+          media->genre            = category;
+          media->isFavorite       = false;
+          media->isPlaying        = false;
+          media->isBroken         = false;
+          media->isPlayed         = false;
+          media->isStopAfter      = false;
 
-          XspfStreams* xspfstream = static_cast<XspfStreams*>(m_services[VIEW::ViewFavoriteRadio]);
-          xspfstream->updateItem(media);
-          xspfstream->saveToFile();
-          xspfstream->reload();
+          FavoriteStreams* favMngr = static_cast<FavoriteStreams*>(m_services[VIEW::ViewFavoriteRadio]);
+          favMngr->updateItem(media);
+          favMngr->saveToDatabase();
+          favMngr->reload();
       }
-      else {
-          StatusWidget::instance()->startShortMessage("please fill all requested fields", STATUS::TYPE_WARNING, 5000);
+      else 
+      {
+          StatusManager::instance()->startMessage("please fill all requested fields", STATUS::WARNING, 5000);
       }      
     }
 }
@@ -853,14 +955,14 @@ void StreamScene::slot_on_import_stream_clicked()
         QList<MEDIA::TrackPtr> streams = MEDIA::PlaylistFromFile(filename);
     
         /* add to favorite model */
-        XspfStreams* xspfstream = static_cast<XspfStreams*>(m_services[VIEW::ViewFavoriteRadio]);
+        FavoriteStreams* favMngr = static_cast<FavoriteStreams*>(m_services[VIEW::ViewFavoriteRadio]);
 
         foreach(MEDIA::TrackPtr stream, streams)
-          xspfstream->updateItem(stream);
+          favMngr->updateItem(stream);
     
-        xspfstream->saveToFile();
-        xspfstream->reload();    
-    }      
+        favMngr->saveToDatabase();
+        favMngr->reload();    
+    }
 }
 
 
@@ -874,9 +976,9 @@ void StreamScene::editStream()
     MEDIA::TrackPtr stream = MEDIA::TrackPtr::staticCast(item->media);
 
     AddStreamDialog dialog(this->parentView(),true);
-    dialog.setCategory(stream->categorie);
-    dialog.setName(stream->name);
-    dialog.setUrl(stream->url);
+    dialog.setCategory( stream->genre );
+    dialog.setName( stream->extra["station"].toString() );
+    dialog.setUrl( stream->url );
     
     if(dialog.exec() == QDialog::Accepted)
     {
@@ -884,14 +986,14 @@ void StreamScene::editStream()
       const QString name     = dialog.name();
       const QString url      = dialog.url();
 
-      stream->url         = url;
-      stream->name        = !name.isEmpty() ? name : url ;
-      stream->categorie   = category;
+      stream->url                = url;
+      stream->extra["station"]   = !name.isEmpty() ? name : url ;
+      stream->genre              = category;
 
-      XspfStreams* xspfstream = static_cast<XspfStreams*>(m_services[VIEW::ViewFavoriteRadio]);
+      FavoriteStreams* favMngr = static_cast<FavoriteStreams*>(m_services[VIEW::ViewFavoriteRadio]);
 
-      xspfstream->saveToFile();
-      xspfstream->reload();
+      favMngr->saveToDatabase();
+      favMngr->reload();
     }
 }
 
